@@ -15,12 +15,11 @@ from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, cast
 
 import boto3
 
-from redun.config import create_config_section
 from redun.executors import aws_utils
 from redun.executors.base import Executor, register_executor
 from redun.executors.code_packaging import package_code, parse_code_package_config
 from redun.executors.command import get_oneshot_command, get_script_task_command
-from redun.executors.docker import DockerExecutor
+from redun.executors.docker import DockerExecutor, get_docker_executor_config
 from redun.executors.scratch import (
     SCRATCH_ERROR,
     SCRATCH_HASHES,
@@ -38,7 +37,7 @@ from redun.file import File
 from redun.job_array import JobArrayer
 from redun.scheduler import Job, Scheduler, Traceback
 from redun.scripting import get_task_command
-from redun.task import Task
+from redun.task import CacheScope, Task
 from redun.utils import json_cache_key, lru_cache_custom, merge_dicts, pickle_dump
 
 SUBMITTED = "SUBMITTED"
@@ -66,6 +65,7 @@ ARRAY_JOB_SUFFIX = "array"
 DOCKER_INSPECT_ERROR = "CannotInspectContainerError: Could not transition to inspecting"
 BATCH_JOB_TIMEOUT_ERROR = "Job attempt duration exceeded timeout"
 DEBUG_SCRATCH = "scratch"
+JOB_ROLE_NONE = "none"
 
 
 def get_default_registry() -> str:
@@ -93,7 +93,6 @@ def get_or_create_repo(repo_name: str) -> dict:
 
 
 def ecr_push(repo_name: str, registry: Optional[str] = None) -> None:
-
     # Get default registry based on login.
     if registry is None:
         registry = get_default_registry()
@@ -163,19 +162,24 @@ def get_job_details(
     container_props = {
         "command": command,
         "image": image,
-        "vcpus": vcpus,
-        "memory": memory,
         "jobRoleArn": role,
         "environment": [],
         "mountPoints": [],
         "volumes": [],
-        "resourceRequirements": [],
+        "resourceRequirements": [
+            {"type": "VCPU", "value": str(vcpus)},
+            {"type": "MEMORY", "value": str(memory * 1024)},  # Convert Gb to Mb.
+        ],
         "ulimits": [],
         "privileged": privileged,
     }
     if shared_memory is not None:
         container_props["linuxParameters"] = {"sharedMemorySize": int(shared_memory * 1024)}
 
+    # Don't specify a role if it's None.
+    if role.lower() == JOB_ROLE_NONE:
+        container_props.pop("jobRoleArn")
+
     if num_nodes is None:
         # Single-node job type
         return {"type": "container", "containerProperties": container_props}
@@ -185,13 +189,7 @@ def get_job_details(
 
         # for multi-node jobs, increase ulimit for number of open file descriptors to
         # allow for large number of open socket connections
-        container_props["ulimits"] = [
-            {
-                "name": "nofile",
-                "softLimit": 65535,
-                "hardLimit": 65535,
-            }
-        ]
+        container_props["ulimits"] = [{"name": "nofile", "softLimit": 65535, "hardLimit": 65535}]
         return {
             "type": "multinode",
             "nodeProperties": {
@@ -200,14 +198,8 @@ def get_job_details(
                 "nodeRangeProperties": [
                     # Create two identical node groups, so we can tell only the main node to
                     # provide outputs.
-                    {
-                        "container": container_props,
-                        "targetNodes": "0",
-                    },
-                    {
-                        "container": container_props,
-                        "targetNodes": "1:",
-                    },
+                    {"container": container_props, "targetNodes": "0"},
+                    {"container": container_props, "targetNodes": "1:"},
                 ],
             },
         }
@@ -222,8 +214,6 @@ def equiv_job_def(job_def1: dict, job_def2: dict) -> bool:
     # At submission-time we override resource properties, so ignore them when
     # looking for an equivalent job def.
     no_resource_container_properties = {
-        "vcpus": "any_vcpus",
-        "memory": "any_memory",
         "resourceRequirements": ["any_requirements"],
     }
 
@@ -231,7 +221,7 @@ def equiv_job_def(job_def1: dict, job_def2: dict) -> bool:
         """Overwrite the resource properties with redactions."""
         result = copy.deepcopy(job_def)
 
-        # Ignore job definition name.
+        # Ignore some fields.
         result.pop("jobDefinitionName", None)
 
         result.setdefault("containerProperties", {}).update(no_resource_container_properties)
@@ -245,7 +235,21 @@ def equiv_job_def(job_def1: dict, job_def2: dict) -> bool:
     # Limit equality to the keys of job_def1.
     job_def1 = sanitize_job_def(job_def1)
     job_def2 = sanitize_job_def(job_def2)
-    return all(job_def1[key] == job_def2[key] for key in job_def1.keys())
+
+    def dict_eq_lhs_keys(lhs: Dict, rhs: Dict) -> bool:
+        """Recursively check that the lhs is a subset of the rhs - Batch may return more keys
+        than we set, usually to empty values, but we don't care."""
+
+        try:
+            return all(
+                lhs[key] == rhs[key]
+                or (isinstance(lhs[key], dict) and dict_eq_lhs_keys(lhs[key], rhs[key]))
+                for key in lhs.keys()
+            )
+        except KeyError:  # key not in rhs.
+            return False
+
+    return dict_eq_lhs_keys(job_def1, job_def2)
 
 
 def get_job_def_revision(job_def_name: str) -> int:
@@ -346,13 +350,9 @@ def create_job_override_command(
         # Single-node jobs override the container.
         batch_job_args["containerOverrides"] = {"command": command}
     else:
-
         # Make a shallow copy so we can suppress output on these nodes
         node_overrides = [
-            {
-                "targetNodes": "0",
-                "containerOverrides": {"command": command},
-            },
+            {"targetNodes": "0", "containerOverrides": {"command": command}},
             {"targetNodes": "1:", "containerOverrides": {"command": command_worker}},
         ]
 
@@ -379,10 +379,12 @@ def batch_submit(
     job_def_extra: Optional[dict] = None,
     aws_region: str = aws_utils.DEFAULT_AWS_REGION,
     privileged: bool = False,
-    autocreate_job: bool = True,
+    autocreate_job_def: bool = True,
     timeout: Optional[int] = None,
     batch_tags: Optional[Dict[str, str]] = None,
     propagate_tags: bool = True,
+    share_id: Optional[str] = None,
+    scheduling_priority_override: Optional[str] = None,
 ) -> Dict[str, Any]:
     """Actually perform job submission to AWS batch. Create or retrieve the job definition, then
     use it to submit the job.
@@ -398,7 +400,7 @@ def batch_submit(
     # Get or create job definition. If autocreate_job is disabled, then we require a job_def_name
     # to be present and lookup the job by name. If autocreate_job is enabled, then we will create
     # a job if an existing one matching job def name and required properties cannot be found.
-    if not autocreate_job:
+    if not autocreate_job_def:
         assert job_def_name
         job_def = get_job_definition(job_def_name, aws_region=aws_region)
         if not job_def:
@@ -422,9 +424,20 @@ def batch_submit(
         # Switch units, redun configs are in GB but AWS uses MB.
         memory_mb = int(memory * 1024)
 
-        container_properties.update({"vcpus": vcpus, "memory": memory_mb})
+        container_properties["resourceRequirements"] = [
+            {
+                "type": "VCPU",
+                "value": str(vcpus),
+            },
+            {
+                "type": "MEMORY",
+                "value": str(memory_mb),
+            },
+        ]
         if gpus > 0:
-            container_properties["resourceRequirements"] = [{"type": "GPU", "value": str(gpus)}]
+            container_properties["resourceRequirements"].append(
+                {"type": "GPU", "value": str(gpus)}
+            )
 
     if "containerOverrides" in batch_job_args:
         apply_resources(batch_job_args["containerOverrides"])
@@ -443,6 +456,12 @@ def batch_submit(
     if batch_tags:
         batch_job_args["tags"] = batch_tags
 
+    # If provided, set batch args required by queues with Scheduling Policies
+    if share_id is not None:
+        batch_job_args["shareIdentifier"] = share_id
+    if scheduling_priority_override is not None:
+        batch_job_args["schedulingPriorityOverride"] = scheduling_priority_override
+
     # Submit to batch.
     batch_run = batch_client.submit_job(
         jobName=job_name,
@@ -492,6 +511,7 @@ def get_batch_job_options(job_options: dict) -> dict:
     """
     Returns AWS Batch-specific job options from general job options.
     """
+    # These options are usually mirrored in default_task_options for use with the redun.ini.
     keys = [
         "vcpus",
         "gpus",
@@ -501,11 +521,13 @@ def get_batch_job_options(job_options: dict) -> dict:
         "retries",
         "privileged",
         "job_def_name",
-        "autocreate_job",
+        "autocreate_job_def",
         "timeout",
         "batch_tags",
         "num_nodes",
         "job_def_extra",
+        "share_id",
+        "scheduling_priority_override",
     ]
     return {key: job_options[key] for key in keys if key in job_options}
 
@@ -544,7 +566,7 @@ def submit_task(
         args,
         kwargs,
         # Suppress cache checking since output is discarded.
-        job_options={**job_options, "cache": False},
+        job_options={**job_options, "cache_scope": CacheScope.NONE},
         code_file=code_file,
         array_uuid=array_uuid,
         output_path="/dev/null",  # Let main command write to scratch file.
@@ -558,7 +580,9 @@ def submit_task(
 
     # Submit to AWS Batch.
     job_name = get_batch_job_name(
-        job_options.get("job_name_prefix", "batch-job"), job_hash, array=bool(array_size)
+        job_options.get("job_name_prefix", "batch-job"),
+        job_hash,
+        array=bool(array_size),
     )
 
     job_batch_options = get_batch_job_options(job_options)
@@ -593,12 +617,7 @@ def submit_command(
     """
     Submit a shell command to AWS Batch.
     """
-    shell_command = get_script_task_command(
-        s3_scratch_prefix,
-        job,
-        command,
-        exit_command="exit 1",
-    )
+    shell_command = get_script_task_command(s3_scratch_prefix, job, command, exit_command="exit 1")
 
     # Submit to AWS Batch.
     assert job.eval_hash
@@ -606,8 +625,7 @@ def submit_command(
 
     job_batch_options = get_batch_job_options(job_options)
     job_batch_args = create_job_override_command(
-        command=shell_command,
-        num_nodes=job_batch_options.get("num_nodes", None),
+        command=shell_command, num_nodes=job_batch_options.get("num_nodes", None)
     )
 
     # Submit to AWS Batch.
@@ -628,8 +646,6 @@ def parse_job_error(
     """
     Parse task error from s3 scratch path.
     """
-    assert job.task
-
     error, error_traceback = _parse_job_error(s3_scratch_prefix, job)
 
     # Handle AWS Batch-specific errors.
@@ -666,7 +682,9 @@ def parse_job_logs(
 
 
 def aws_describe_jobs(
-    job_ids: List[str], chunk_size: int = 100, aws_region: str = aws_utils.DEFAULT_AWS_REGION
+    job_ids: List[str],
+    chunk_size: int = 100,
+    aws_region: str = aws_utils.DEFAULT_AWS_REGION,
 ) -> Iterator[dict]:
     """
     Returns AWS Batch Job descriptions from the AWS API.
@@ -682,7 +700,9 @@ def aws_describe_jobs(
 
 
 def iter_batch_job_status(
-    job_ids: List[str], pending_truncate: int = 10, aws_region: str = aws_utils.DEFAULT_AWS_REGION
+    job_ids: List[str],
+    pending_truncate: int = 10,
+    aws_region: str = aws_utils.DEFAULT_AWS_REGION,
 ) -> Iterator[dict]:
     """
     Yields AWS Batch jobs statuses.
@@ -802,14 +822,6 @@ class AWSBatchJobTimeoutError(Exception):
     pass
 
 
-def get_docker_executor_config(config: SectionProxy) -> SectionProxy:
-    """
-    Returns a config for DockerExecutor.
-    """
-    keys = ["image", "scratch", "job_monitor_interval", "vcpus", "gpus", "memory"]
-    return create_config_section({key: config[key] for key in keys if key in config})
-
-
 def parse_nullable_json(text: Optional[str]) -> Any:
     if text is None:
         return None
@@ -851,16 +863,27 @@ class AWSBatchExecutor(Executor):
         self.debug = config.getboolean("debug", fallback=False)
 
         # Default task options.
+        # Defaults should be set for all values mirrored in get_batch_job_options.
         self.default_task_options: Dict[str, Any] = {
             "vcpus": config.getint("vcpus", fallback=1),
             "gpus": config.getint("gpus", fallback=0),
-            "memory": config.getint("memory", fallback=4),
+            "memory": config.getfloat("memory", fallback=4),
             "shared_memory": config.getint("shared_memory", fallback=None),
+            "timeout": config.getint("timeout", fallback=None),
+            "privileged": config.getboolean("privileged", fallback=False),
+            "autocreate_job_def": config.getboolean(
+                "autocreate_job_def",
+                # autocreate_job is a deprecated but backwards compatible alternative.
+                fallback=config.getboolean("autocreate_job", fallback=True),
+            ),
+            "job_def_name": config.get("job_def_name", fallback=None),
             "retries": config.getint("retries", fallback=1),
             "role": config.get("role"),
             "job_name_prefix": config.get("job_name_prefix", fallback="redun-job"),
             "num_nodes": config.getint("num_nodes", fallback=None),
             "job_def_extra": parse_nullable_json(config.get("job_def_extra", fallback=None)),
+            "share_id": config.get("share_id"),
+            "scheduling_priority_override": config.getint("scheduling_priority_override"),
         }
         if config.get("batch_tags"):
             self.default_task_options["batch_tags"] = json.loads(config.get("batch_tags"))
@@ -874,7 +897,8 @@ class AWSBatchExecutor(Executor):
 
         self._thread: Optional[threading.Thread] = None
         self.arrayer = JobArrayer(
-            executor=self,
+            self._submit_jobs,
+            self._on_error,
             submit_interval=self.interval,
             stale_time=config.getfloat("job_stale_time", fallback=3.0),
             min_array_size=config.getint("min_array_size", fallback=5),
@@ -882,12 +906,28 @@ class AWSBatchExecutor(Executor):
         )
         self._aws_user: Optional[str] = None
 
+    def _submit_jobs(self, jobs: List[Job]) -> None:
+        """
+        Callback for JobArrayer to return arrays of Jobs.
+        """
+        if len(jobs) == 1:
+            # Singleton jobs are given as single element lists.
+            self._submit_single_job(jobs[0])
+        else:
+            self._submit_array_job(jobs)
+
+    def _on_error(self, error: Exception) -> None:
+        """
+        Callback for JobArrayer to report scheduler-level errors.
+        """
+        assert self._scheduler
+        self._scheduler.reject_job(None, error)
+
     def set_scheduler(self, scheduler: Scheduler) -> None:
         super().set_scheduler(scheduler)
         self._docker_executor.set_scheduler(scheduler)
 
     def gather_inflight_jobs(self) -> None:
-
         running_arrays: Dict[str, List[Tuple[str, int]]] = defaultdict(list)
 
         # Get all running jobs by name
@@ -912,7 +952,6 @@ class AWSBatchExecutor(Executor):
 
         # Match up running array jobs with consistent redun job naming scheme.
         for array_name, child_job_indices in running_arrays.items():
-
             # Get path to array file directory on S3 from array job name.
             parent_hash = get_hash_from_job_name(array_name)
             if not parent_hash:
@@ -1036,7 +1075,6 @@ class AWSBatchExecutor(Executor):
 
         if DOCKER_INSPECT_ERROR in container_reason:
             redun_job = self.pending_batch_jobs[job["jobId"]]
-            assert redun_job.task
             if redun_job.task.script:
                 # Script tasks will report their status in a status file.
                 status_file = File(
@@ -1126,14 +1164,13 @@ class AWSBatchExecutor(Executor):
         Task options can be specified at the job-level have precedence over
         the executor-level (within `redun.ini`):
         """
-        assert job.task
-
         job_options = job.get_options()
 
-        task_options = {
-            **self.default_task_options,
-            **job_options,
-        }
+        # Normalize deprecated options.
+        if "autocreate_job" in job_options and "autocreate_job_def" not in job_options:
+            job_options["autocreate_job_def"] = job_options["autocreate_job"]
+
+        task_options = {**self.default_task_options, **job_options}
 
         # Add default batch tags to the job.
         if self.use_default_batch_tags:
@@ -1164,12 +1201,11 @@ class AWSBatchExecutor(Executor):
 
         return task_options
 
-    def _submit(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def _submit(self, job: Job) -> None:
         """
         Submit Job to executor.
         """
         assert self._scheduler
-        assert job.task
         assert not self.debug
 
         # If we are not in debug mode and this is the first submission gather inflight jobs. In
@@ -1195,11 +1231,11 @@ class AWSBatchExecutor(Executor):
 
         # Determine job options.
         task_options = self._get_job_options(job)
-        use_cache = task_options.get("cache", True)
+        cache_scope = CacheScope(task_options.get("cache_scope", CacheScope.BACKEND))
 
         # Determine if we can reunite with a previous Batch output or job.
         batch_job_id: Optional[str] = None
-        if use_cache and job.eval_hash in self.preexisting_batch_jobs:
+        if cache_scope == CacheScope.BACKEND and job.eval_hash in self.preexisting_batch_jobs:
             batch_job_id = self.preexisting_batch_jobs.pop(job.eval_hash)
 
             # Make sure Batch API still has a status on this job.
@@ -1227,20 +1263,22 @@ class AWSBatchExecutor(Executor):
         # Job arrayer will handle actual submission after bunching to an array
         # job, if necessary.
         if batch_job_id is None:
-            self.arrayer.add_job(job, args, kwargs)
+            self.arrayer.add_job(job)
 
         self._start()
 
-    def _submit_array_job(
-        self, jobs: List[Job], all_args: List[Tuple], all_kwargs: List[Dict]
-    ) -> str:
+    def _submit_array_job(self, jobs: List[Job]) -> str:
         """Submits an array job, returning job name uuid"""
         array_size = len(jobs)
-        assert array_size == len(all_args) == len(all_kwargs)
+        all_args = []
+        all_kwargs = []
+        for job in jobs:
+            assert job.args
+            all_args.append(job.args[0])
+            all_kwargs.append(job.args[1])
 
         # All jobs identical so just grab the first one
         job = jobs[0]
-        assert job.task
         if job.task.script:
             raise NotImplementedError("Array jobs not supported for scripts")
 
@@ -1314,14 +1352,21 @@ class AWSBatchExecutor(Executor):
             )
         )
 
+        # AWS Batch job tags at submission time.
+        assert self._scheduler
+        for i, job in enumerate(jobs):
+            self._scheduler.add_job_tags(job, [("aws_batch_job", f"{array_job_id}:{i}")])
+
         return array_uuid
 
-    def _submit_single_job(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def _submit_single_job(self, job) -> None:
         """
         Actually submits a job. Caching detects if it should be part
         of an array job
         """
-        assert job.task
+        assert job.args
+        args, kwargs = job.args
+
         task_options = self._get_job_options(job)
         image = task_options.pop("image", self.image)
         queue = task_options.pop("queue", self.queue)
@@ -1372,29 +1417,33 @@ class AWSBatchExecutor(Executor):
         batch_job_id = batch_resp["jobId"]
         self.pending_batch_jobs[batch_job_id] = job
 
+        # Add batch job tags at submission time.
+        assert self._scheduler
+        self._scheduler.add_job_tags(job, [("aws_batch_job", batch_job_id)])
+
     def _is_debug_job(self, job: Job) -> bool:
         """
         Returns True if job should be sent to debugging DockerExecutor.
         """
         return self.debug or job.get_option("debug", False)
 
-    def submit(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def submit(self, job: Job) -> None:
         """
         Submit Job to executor.
         """
         if self._is_debug_job(job):
-            return self._docker_executor.submit(job, args, kwargs)
+            return self._docker_executor.submit(job)
         else:
-            return self._submit(job, args, kwargs)
+            return self._submit(job)
 
-    def submit_script(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def submit_script(self, job: Job) -> None:
         """
         Submit Job for script task to executor.
         """
         if self._is_debug_job(job):
-            return self._docker_executor.submit_script(job, args, kwargs)
+            return self._docker_executor.submit_script(job)
         else:
-            return self._submit(job, args, kwargs)
+            return self._submit(job)
 
     def get_jobs(self, statuses: Optional[List[str]] = None) -> Iterator[dict]:
         """
@@ -1437,3 +1486,6 @@ class AWSBatchExecutor(Executor):
 
         for job_id in job_ids:
             yield batch_client.terminate_job(jobId=job_id, reason=reason)
+
+    def scratch_root(self) -> str:
+        return self.s3_scratch_prefix
