@@ -8,7 +8,11 @@ from typing import Any, Deque, Dict, Iterator, List, Optional, Tuple, Union, cas
 
 from redun.executors import aws_utils
 from redun.executors.base import Executor, register_executor
-from redun.executors.code_packaging import create_zip, package_code, parse_code_package_config
+from redun.executors.code_packaging import (
+    create_zip,
+    package_code,
+    parse_code_package_config,
+)
 from redun.executors.command import REDUN_PROG, REDUN_REQUIRED_VERSION
 from redun.executors.scratch import (
     SCRATCH_ERROR,
@@ -23,7 +27,7 @@ from redun.executors.scratch import parse_job_result
 from redun.file import File
 from redun.hashing import hash_stream, hash_text
 from redun.scheduler import Job, Scheduler, Traceback
-from redun.task import Task
+from redun.task import CacheScope, Task
 from redun.utils import pickle_dump
 
 ARGS = ["code", "script", "task", "input", "output", "error"]
@@ -32,7 +36,16 @@ ONESHOT_FILE = "glue_oneshot.py.txt"
 
 # AWS Glue job statuses
 GLUE_JOB_STATUSES = aws_utils.JobStatus(
-    all=["STARTING", "RUNNING", "STOPPING", "SUCCEEDED", "FAILED", "ERROR", "STOPPED", "TIMEOUT"],
+    all=[
+        "STARTING",
+        "RUNNING",
+        "STOPPING",
+        "SUCCEEDED",
+        "FAILED",
+        "ERROR",
+        "STOPPED",
+        "TIMEOUT",
+    ],
     inflight=["STARTING", "RUNNING", "STOPPING"],
     pending=["STARTING"],
     success=["SUCCEEDED"],
@@ -48,7 +61,7 @@ DEFAULT_ADDITIONAL_PYTHON_MODULES = [
     "mako",
     "promise",
     "s3fs>=2021.11.1",
-    "sqlalchemy>=1.3.17or<2",  # Don't use commas as AWS won't parse them.
+    "sqlalchemy>=1.4.0or<2.1",  # Don't use commas as AWS won't parse them.
 ]
 
 
@@ -108,6 +121,7 @@ def package_redun_lib(s3_scratch_prefix: str) -> File:
 def get_or_create_glue_job_definition(
     script_location: str,
     role: str,
+    glue_version: str,
     temp_dir: str,
     extra_py_files: str,
     spark_history_dir: str,
@@ -125,6 +139,9 @@ def get_or_create_glue_job_definition(
     role : str
         ARN of IAM role to associate with Glue job.
 
+    glue_version : str
+        Glue version to use for the job.
+
     temp_dir : str
         S3 path of scratch directory associated with job data and code.
 
@@ -173,7 +190,7 @@ def get_or_create_glue_job_definition(
         MaxRetries=0,
         NumberOfWorkers=2,  # Jobs will override this, so set to minimum value.
         WorkerType="Standard",
-        GlueVersion="3.0",
+        GlueVersion=glue_version,
         Timeout=2880,
     )
     glue_job_def_hash = hash_text(json.dumps(glue_job_def, sort_keys=True))
@@ -218,6 +235,7 @@ class AWSGlueExecutor(Executor):
 
         # Optional config
         self.aws_region = config.get("aws_region", aws_utils.get_default_region())
+        self.glue_version = config.get("glue_version", "4.0")
         self.role = config.get("role") or get_default_glue_service_role(aws_region=self.aws_region)
         self.code_package = parse_code_package_config(config)
         self.code_file: Optional[File] = None
@@ -244,7 +262,7 @@ class AWSGlueExecutor(Executor):
         self.is_running = False
         self._monitor_thread = threading.Thread(target=self._monitor, daemon=False)
         self._submit_thread = threading.Thread(target=self._submission_thread, daemon=False)
-        self.pending_glue_jobs: Deque[Tuple["Job", Tuple, Dict]] = deque()
+        self.pending_glue_jobs: Deque["Job"] = deque()
         self.running_glue_jobs: Dict[str, "Job"] = OrderedDict()
         self.preexisting_glue_jobs: Dict[str, str] = {}  # Job hash -> Job ID
         self._oneshot_path: Optional[str] = None
@@ -296,6 +314,7 @@ class AWSGlueExecutor(Executor):
             temp_dir=self.s3_scratch_prefix,
             extra_py_files=self.redun_zip_location,
             aws_region=self.aws_region,
+            glue_version=self.glue_version,
         )
 
     def _start(self) -> None:
@@ -353,12 +372,12 @@ class AWSGlueExecutor(Executor):
             while self.is_running and self.pending_glue_jobs:
                 fail_counter = 0
                 while fail_counter < 5 and self.pending_glue_jobs:
-                    job, args, kwargs = self.pending_glue_jobs.popleft()
+                    job = self.pending_glue_jobs.popleft()
                     job_id = self.submit_pending_job(job)
 
                     if job_id is None:
                         fail_counter += 1
-                        self.pending_glue_jobs.append((job, args, kwargs))
+                        self.pending_glue_jobs.append(job)
                     else:
                         self.running_glue_jobs[job_id] = job
                         fail_counter = 0
@@ -430,18 +449,16 @@ class AWSGlueExecutor(Executor):
         """
         Determines task options for a job.
         """
-        assert job.task
-
         task_options = dict(self.default_task_options)
         task_options.update(job.get_options())
         return task_options
 
-    def submit(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def submit(self, job: Job) -> None:
         """
         Submit job to executor.
         """
         assert self._scheduler
-        assert job.task
+        assert job.args
 
         # Check glue job definition exists. Otherwise, create it.
         if not self.glue_job_name:
@@ -466,15 +483,18 @@ class AWSGlueExecutor(Executor):
 
         # Determine if we can reunite with a previous Glue output or job.
         glue_job_id: Optional[str] = None
-        use_cache = task_options.get("cache", True)
-        if use_cache and job.eval_hash in self.preexisting_glue_jobs:
+        cache_scope = CacheScope(task_options.get("cache_scope", CacheScope.BACKEND))
+
+        if cache_scope == CacheScope.BACKEND and job.eval_hash in self.preexisting_glue_jobs:
             assert self.glue_job_name
             glue_job_id = self.preexisting_glue_jobs.pop(job.eval_hash)
 
             # Make sure glue API still has a status on this job
             existing_job = next(
                 glue_describe_jobs(
-                    [glue_job_id], glue_job_name=self.glue_job_name, aws_region=self.aws_region
+                    [glue_job_id],
+                    glue_job_name=self.glue_job_name,
+                    aws_region=self.aws_region,
                 )
             )
 
@@ -495,9 +515,9 @@ class AWSGlueExecutor(Executor):
             input_path = get_job_scratch_file(self.s3_scratch_prefix, job, SCRATCH_INPUT)
             input_file = File(input_path)
             with input_file.open("wb") as out:
-                pickle_dump([args, kwargs], out)
+                pickle_dump(job.args, out)
 
-            self.pending_glue_jobs.append((job, args, kwargs))
+            self.pending_glue_jobs.append(job)
 
         self._start()
 
@@ -505,7 +525,6 @@ class AWSGlueExecutor(Executor):
         """
         Returns true if job submission was successful
         """
-        assert job.task
         assert self.glue_job_name
         assert self.redun_zip_location
         assert self.code_file
@@ -545,6 +564,9 @@ class AWSGlueExecutor(Executor):
         )
         return glue_resp["JobRunId"]
 
+    def scratch_root(self) -> str:
+        return self.s3_scratch_prefix
+
 
 def submit_glue_job(
     job: Job,
@@ -665,9 +687,10 @@ def submit_glue_job(
 
 
 def glue_describe_jobs(
-    job_ids: List[str], glue_job_name: str, aws_region: str = aws_utils.DEFAULT_AWS_REGION
+    job_ids: List[str],
+    glue_job_name: str,
+    aws_region: str = aws_utils.DEFAULT_AWS_REGION,
 ) -> Iterator[Dict[str, Any]]:
-
     glue_client = aws_utils.get_aws_client("glue", aws_region=aws_region)
 
     for id in job_ids:
@@ -683,14 +706,13 @@ def parse_job_error(
     """
     Parse job error from s3 scratch path.
     """
-    assert job.task
-
     error, error_traceback = _parse_job_error(s3_scratch_prefix, job)
 
     # Handle AWS Batch-specific errors.
     if isinstance(error, ExceptionNotFoundError) and glue_job_metadata:
         message = glue_job_metadata.get(
-            "ErrorMessage", "Exception and traceback could not be found for AWS Glue Job"
+            "ErrorMessage",
+            "Exception and traceback could not be found for AWS Glue Job",
         )
         error = AWSGlueError(message)
         error_traceback = Traceback.from_error(error)
