@@ -1,6 +1,6 @@
 import importlib
 import typing
-from typing import Any, Callable, Dict, Iterator, Optional, Tuple, Type, cast
+from typing import Any, Callable, Dict, Iterator, Optional, Type, cast
 
 if typing.TYPE_CHECKING:
     from redun.scheduler import Job, Scheduler
@@ -33,13 +33,23 @@ class Executor:
         assert self._scheduler
         self._scheduler.log(f"Executor[{self.name}]:", *messages, **kwargs)
 
-    def submit(self, job: "Job", args: Tuple, kwargs: dict) -> None:
+    def submit(self, job: "Job") -> None:
+        """Execute the provided job.
+
+        Implementations must provide results back to the scheduler by either calling `done_job` or
+        `reject_job`."""
         assert self._scheduler
         self._scheduler.reject_job(
-            job, ExecutorError("Executor {} does not support submitting tasks.".format(type(self)))
+            job,
+            ExecutorError("Executor {} does not support submitting tasks.".format(type(self))),
         )
 
-    def submit_script(self, job: "Job", args: Tuple, kwargs: dict) -> None:
+    def submit_script(self, job: "Job") -> None:
+        """Execute the provided script job.
+
+        Implementations must provide results back to the scheduler by either calling `done_job` or
+        `reject_job`."""
+
         assert self._scheduler
         self._scheduler.reject_job(
             job,
@@ -54,6 +64,9 @@ class Executor:
     def stop(self) -> None:
         pass
 
+    def scratch_root(self) -> str:
+        raise NotImplementedError()
+
 
 # Singleton executor registry.
 _executor_classes: Dict[str, Type[Executor]] = {}
@@ -110,6 +123,18 @@ def get_executors_from_config(executors_config: dict) -> Iterator[Executor]:
         yield executor
 
 
+def get_executor_from_config(executors_config: dict, executor_name: str) -> Executor:
+    """Create and return the executor by name. Raise an error if it is not present."""
+    for name, executor_config in executors_config.items():
+        if name != executor_name:
+            continue
+
+        executor_class = cast(Type[Executor], get_executor_class(executor_config["type"]))
+        executor = executor_class(executor_name, config=executor_config)
+        return executor
+    raise RuntimeError(f"Unknown Executor {executor_name}.")
+
+
 def load_task_module(module_name: str, task_name: str) -> None:
     """
     Helper method that Executors may call to load a task's module code.
