@@ -1,3 +1,4 @@
+import json
 import logging
 import os
 import subprocess
@@ -8,6 +9,7 @@ from configparser import SectionProxy
 from tempfile import mkstemp
 from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
 
+from redun.config import create_config_section
 from redun.executors import aws_utils
 from redun.executors.base import Executor, register_executor
 from redun.executors.code_packaging import package_code, parse_code_package_config
@@ -33,6 +35,14 @@ class DockerError(Exception):
     pass
 
 
+def get_docker_executor_config(config: SectionProxy) -> SectionProxy:
+    """
+    Returns a config for DockerExecutor.
+    """
+    keys = ["image", "scratch", "job_monitor_interval", "vcpus", "gpus", "memory"]
+    return create_config_section({key: config[key] for key in keys if key in config})
+
+
 def run_docker(
     command: List[str],
     image: str,
@@ -43,6 +53,7 @@ def run_docker(
     vcpus: int = 1,
     gpus: int = 0,
     shared_memory: Optional[int] = None,
+    include_aws_env: bool = False,
 ) -> str:
     """
     Run a Docker container locally.
@@ -67,20 +78,30 @@ def run_docker(
         Number of GPUs to reserve for the container.
     shared_memory : Optional[int]
         Number of GB of shared memory to reserve for the container.
+    include_aws_env : bool
+        If True, forward AWS environment variables to the container.
     """
     # Add AWS credentials to environment for docker command.
-    env = dict(os.environ)
-    if not aws_utils.is_ec2_instance():
-        env.update(aws_utils.get_aws_env_vars())
-
     common_args = []
     if cleanup:
         common_args.append("--rm")
 
-    # Environment args.
-    common_args.extend(
-        ["-e", "AWS_ACCESS_KEY_ID", "-e", "AWS_SECRET_ACCESS_KEY", "-e", "AWS_SESSION_TOKEN"]
-    )
+    env = dict(os.environ)
+    if include_aws_env and not aws_utils.is_ec2_instance():
+        # Forward AWS environment args to docker container.
+        env.update(aws_utils.get_aws_env_vars())
+        common_args.extend(
+            [
+                "-e",
+                "AWS_ACCESS_KEY_ID",
+                "-e",
+                "AWS_SECRET_ACCESS_KEY",
+                "-e",
+                "AWS_SESSION_TOKEN",
+                "-e",
+                "AWS_DEFAULT_REGION",
+            ]
+        )
 
     # Volume mounting args.
     for host, container in volumes:
@@ -134,7 +155,15 @@ def get_docker_job_options(job_options: dict, scratch_path: str) -> dict:
 
     Adds the scratch_path as a volume mount.
     """
-    keys = ["vcpus", "memory", "gpus", "shared_memory", "volumes", "interactive"]
+    keys = [
+        "vcpus",
+        "memory",
+        "gpus",
+        "shared_memory",
+        "volumes",
+        "interactive",
+        "include_aws_env",
+    ]
     options = {key: job_options[key] for key in keys if key in job_options}
     options["volumes"] = options.get("volumes", []) + [(scratch_path, scratch_path)]
     return options
@@ -149,6 +178,7 @@ def submit_task(
     kwargs: Dict[str, Any] = {},
     job_options: dict = {},
     code_file: Optional[File] = None,
+    include_aws_env: bool = False,
 ) -> Dict[str, Any]:
     """
     Submit a redun Task to Docker.
@@ -176,13 +206,16 @@ def submit_command(
     job: Job,
     command: str,
     job_options: dict = {},
+    include_aws_env: bool = False,
 ) -> dict:
     """
     Submit a shell command to Docker.
     """
     shell_command = get_script_task_command(scratch_prefix, job, command)
     container_id = run_docker(
-        shell_command, image=image, **get_docker_job_options(job_options, scratch_prefix)
+        shell_command,
+        image=image,
+        **get_docker_job_options(job_options, scratch_prefix),
     )
     return {"jobId": container_id, "redun_job_id": job.id}
 
@@ -229,6 +262,7 @@ class DockerExecutor(Executor):
         super().__init__(name, scheduler=scheduler)
         if config is None:
             raise ValueError("DockerExecutor requires config.")
+        self._config = config
 
         # Required config.
         self._image = config["image"]
@@ -241,36 +275,52 @@ class DockerExecutor(Executor):
         self._code_file: Optional[File] = None
 
         # Default task options.
-        self._default_job_options = {
+        self._default_job_options: Dict[str, Any] = {
             "vcpus": config.getint("vcpus", fallback=1),
             "gpus": config.getint("gpus", fallback=0),
             "memory": config.getint("memory", fallback=4),
             "shared_memory": config.getint("shared_memory", fallback=None),
             "interactive": config.getboolean("interactive", fallback=False),
+            "include_aws_env": config.getboolean("include_aws_env", fallback=True),
         }
 
         self._is_running = False
         self._pending_jobs: Dict[str, "Job"] = OrderedDict()
         self._thread: Optional[threading.Thread] = None
 
+    def set_scheduler(self, scheduler: "Scheduler") -> None:
+        super().set_scheduler(scheduler)
+        # Perform additional config parsing now that we know our Scheduler.
+        self._default_job_options["volumes"] = self._parse_volumes(
+            self._config.get("volumes", fallback="[]")
+        )
+
+    def _parse_volumes(self, volumes_str: str) -> List[List[str]]:
+        """
+        Parses Docker volume mounts from JSON string.
+        """
+        try:
+            volumes = json.loads(volumes_str)
+        except json.decoder.JSONDecodeError:
+            raise ValueError("Invalid 'volumes' syntax. Expected JSON list of path pairs.")
+
+        # Convert host_paths to absolute paths if they are relative.
+        # Relative paths are assumed to be relative to the config dir.
+        assert self._scheduler
+        configdir = self._scheduler.config.configdir
+        return [
+            [os.path.abspath(os.path.join(configdir, host_path)), container_path]
+            for host_path, container_path in volumes
+        ]
+
     @property
     def _scratch_prefix(self) -> str:
         if not self._scratch_prefix_abs:
             if os.path.isabs(self._scratch_prefix_rel):
                 self._scratch_prefix_abs = self._scratch_prefix_rel
             else:
-                # TODO: Is there a better way to find the path of the current
-                # config dir?
-                try:
-                    assert self._scheduler
-                    base_dir = os.path.abspath(
-                        self._scheduler.config["repos"]["default"]["config_dir"]
-                    )
-                except KeyError:
-                    # Use current working directory as base_dir if default
-                    # config_dir cannot be found.
-                    base_dir = os.getcwd()
-
+                assert self._scheduler
+                base_dir = os.path.abspath(self._scheduler.config.configdir)
                 self._scratch_prefix_abs = os.path.normpath(
                     os.path.join(base_dir, self._scratch_prefix_rel)
                 )
@@ -351,12 +401,13 @@ class DockerExecutor(Executor):
             error_traceback.logs = [line + "\n" for line in job["logs"].split("\n")]
             self._scheduler.reject_job(redun_job, error, error_traceback=error_traceback)
 
-    def _submit(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def _submit(self, job: Job) -> None:
         """
         Submit Job to executor.
         """
         assert self._scheduler
-        assert job.task
+        assert job.args
+        args, kwargs = job.args
 
         # Package code if necessary and we have not already done so. If code_package is False,
         # then we can skip this step. Additionally, if we have already packaged and set code_file,
@@ -412,14 +463,17 @@ class DockerExecutor(Executor):
         self._pending_jobs[docker_resp["jobId"]] = job
         self._start()
 
-    def submit(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def submit(self, job: Job) -> None:
         """
         Submit Job to executor.
         """
-        return self._submit(job, args, kwargs)
+        return self._submit(job)
 
-    def submit_script(self, job: Job, args: Tuple, kwargs: dict) -> None:
+    def submit_script(self, job: Job) -> None:
         """
         Submit Job for script task to executor.
         """
-        return self._submit(job, args, kwargs)
+        return self._submit(job)
+
+    def scratch_root(self) -> str:
+        return self._scratch_prefix
