@@ -74,6 +74,7 @@ class LocalExecutor(Executor):
         self.mode = config.get("mode", mode)
         # Autoconvert deprecated modes.
         self.mode = self._OLD2NEW_MODES.get(self.mode, self.mode)
+        self._scratch_root = config.get("scratch", "/tmp/redun")
         assert self.mode in self.MODES, f"Unknown mode: {self.mode}"
 
         self.start_method = config.get("start_method", self.DEFAULT_START_METHOD)
@@ -103,7 +104,8 @@ class LocalExecutor(Executor):
                 self._process_executor = ProcessPoolExecutor(max_workers=self.max_workers)
             else:
                 self._process_executor = ProcessPoolExecutor(
-                    max_workers=self.max_workers, mp_context=get_context(self.start_method)
+                    max_workers=self.max_workers,
+                    mp_context=get_context(self.start_method),
                 )
 
     def stop(self) -> None:
@@ -121,7 +123,7 @@ class LocalExecutor(Executor):
                 self._process_executor.shutdown()
             self._process_executor = None
 
-    def _submit(self, exec_func: Callable, job: "Job", args: Tuple, kwargs: dict) -> None:
+    def _submit(self, exec_func: Callable, job: "Job") -> None:
         mode = job.get_option("mode", self.mode)
         if mode not in (THREAD_MODE, PROCESS_MODE):
             raise ValueError(f"Unknown mode: {mode}")
@@ -141,7 +143,6 @@ class LocalExecutor(Executor):
             raise ValueError('Unknown LocalExecutor.mode "{}"'.format(mode))
 
         # Run job in a new thread or process.
-        assert job.task
 
         def on_done(future):
             try:
@@ -149,16 +150,19 @@ class LocalExecutor(Executor):
             except Exception as error:
                 self._scheduler.reject_job(job, error)
 
+        assert job.args
+        args, kwargs = job.args
         executor.submit(
             exec_func, mode, job.task.load_module, job.task.fullname, args, kwargs
         ).add_done_callback(on_done)
 
-    def submit(self, job: "Job", args: Tuple, kwargs: dict) -> None:
-        assert job.task
+    def submit(self, job: "Job") -> None:
         assert not job.task.script
-        self._submit(exec_task, job, args, kwargs)
+        self._submit(exec_task, job)
 
-    def submit_script(self, job: "Job", args: Tuple, kwargs: dict) -> None:
-        assert job.task
+    def submit_script(self, job: "Job") -> None:
         assert job.task.script
-        self._submit(exec_script_task, job, args, kwargs)
+        self._submit(exec_script_task, job)
+
+    def scratch_root(self) -> str:
+        return self._scratch_root
