@@ -5,6 +5,7 @@ Glue related imports are kept isolated, so this can be imported
 people's local machines without issue, and then they call the
 functions that will run when the context is defined.
 """
+
 import functools
 import typing
 from typing import Callable, List, Optional
@@ -18,7 +19,6 @@ if typing.TYPE_CHECKING:
 
 
 def setup_glue_job(job_name: str, job_args: List[str]) -> "GlueJob":
-
     try:
         from awsglue.context import GlueContext
         from awsglue.job import Job as GlueJob
@@ -89,7 +89,9 @@ def get_num_workers() -> int:
     return int(workers)
 
 
-def udf(wrapped_func: Callable = None, return_type: Optional["DataType"] = None) -> Callable:
+def udf(
+    wrapped_func: Optional[Callable] = None, return_type: Optional["DataType"] = None
+) -> Callable:
     """
     Creates a spark user defined function.
     Wraps `pyspark.sql.functions.udf` so spark context is only needed at
