@@ -1,6 +1,8 @@
+import enum
 import inspect
 import re
 import sys
+import typing
 from typing import (
     Any,
     Callable,
@@ -17,17 +19,51 @@ from typing import (
 )
 
 from redun.expression import SchedulerExpression, TaskExpression
-from redun.hashing import hash_arguments, hash_struct
-from redun.namespace import get_current_namespace
+from redun.hashing import hash_arguments, hash_eval, hash_struct
+from redun.namespace import compute_namespace
 from redun.promise import Promise
-from redun.utils import get_func_source
-from redun.value import Value, get_type_registry
+from redun.utils import get_func_source, merge_dicts
+from redun.value import TypeRegistry, Value, get_type_registry
 
+T = TypeVar("T")
 Func = TypeVar("Func", bound=Callable)
 Func2 = TypeVar("Func2", bound=Callable)
 Result = TypeVar("Result")
 
 
+class CacheCheckValid(enum.Enum):
+    """
+    Types of validity checking for cache hits.
+    """
+
+    FULL = "full"  # Recursively check all intermediate values in the call tree
+    SHALLOW = "shallow"  # Only check the final result for validity
+
+
+class CacheScope(enum.Enum):
+    """
+    Types of cache hits and misses.
+    """
+
+    NONE = "NONE"  # Do not cache
+    CSE = "CSE"  # Only allow cache hits against tasks from this execution
+    BACKEND = "BACKEND"  # Allow cache hits from any execution known to the backend
+
+
+class CacheResult(enum.Enum):
+    """
+    Types of cache hits and misses.
+    """
+
+    CSE = "CSE"  # Common Subexpression Elimination hit, i.e., an ultimate reduction from this
+    # execution
+    SINGLE = "SINGLE"  # The result from a single reduction, from a different execution,
+    # i.e., an evaluation cache hit
+    ULTIMATE = "ULTIMATE"  # The ultimate result from the expression, from a different execution,
+    # i.e., a call cache hit
+    MISS = "MISS"  # Complete cache miss.
+
+
 def get_task_registry():
     """
     Returns the global task registry.
@@ -67,12 +103,18 @@ def get_tuple_type_length(tuple_type: Any) -> Optional[int]:
         else:
             tuple_type_args = tuple_type.__args__
 
-        if Ellipsis in tuple_type_args or len(tuple_type_args) == 0:
+        if Ellipsis in tuple_type_args or tuple_type is Tuple:
             # Tuple of unknown length.
             return None
-
-        if tuple_type_args == ((),):
+        elif tuple_type_args == ((),):
             # Special case for length zero.
+            # Note that the representation of empty tuple types changed in python 3.11
+            # Specifically: get_args(Tuple[()]) now evaluates to () instead of ((),)
+            # which is the same value that get_args(Tuple) evaluates to.
+            # https://docs.python.org/3/whatsnew/3.11.html
+            #
+            # Because of this we need to check for the type `Tuple` above before entering
+            # this branch.
             return 0
 
         return len(tuple_type_args)
@@ -86,6 +128,17 @@ def get_tuple_type_length(tuple_type: Any) -> Optional[int]:
     return None
 
 
+def split_task_fullname(task_fullname: str) -> Tuple[str, str]:
+    """
+    Split a Task fullname into a namespace and name.
+    """
+    if "." in task_fullname:
+        namespace, name = task_fullname.rsplit(".", 1)
+        return namespace, name
+    else:
+        return "", task_fullname
+
+
 class Task(Value, Generic[Func]):
     """
     A redun Task.
@@ -134,7 +187,7 @@ class Task(Value, Generic[Func]):
         source: Optional[str] = None,
     ):
         self.name = name or func.__name__
-        self.namespace = self._compute_namespace(func, namespace)
+        self.namespace = compute_namespace(func, namespace)
         self.func = func
         if source is not None:
             self.source = source
@@ -160,26 +213,6 @@ class Task(Value, Generic[Func]):
     def recompute_hash(self):
         self.hash = self._calc_hash()
 
-    @staticmethod
-    def _compute_namespace(func: Callable, namespace: Optional[str] = None):
-        """Compute the namespace for the provided function to wrap.
-
-        WARNING: The computation is stateful and hence this can only be used during task
-        creation.
-
-        Precedence:
-        - Explicit namespace provided (empty string is a valid explicit value)
-        - Infer it from a `redun_namespace` variable in the same module as func
-        - The current namespace, configured with `set_current_namespace`"""
-
-        # Determine task namespace.
-        if namespace is None:
-            namespace = getattr(sys.modules[func.__module__], "redun_namespace", None)
-
-        if namespace is not None:
-            return namespace
-        return get_current_namespace()
-
     @property
     def nout(self) -> Optional[int]:
         """
@@ -198,15 +231,11 @@ class Task(Value, Generic[Func]):
 
         return nout
 
-    T = TypeVar("T")
+    @overload
+    def get_task_option(self, option_name: str) -> Optional[Any]: ...
 
     @overload
-    def get_task_option(self, option_name: str) -> Optional[Any]:
-        ...
-
-    @overload
-    def get_task_option(self, option_name: str, default: T) -> T:
-        ...
+    def get_task_option(self, option_name: str, default: T) -> T: ...
 
     def get_task_option(self, option_name: str, default: Optional[T] = None) -> Optional[T]:
         """
@@ -274,6 +303,19 @@ class Task(Value, Generic[Func]):
             if self.nout < 0:
                 raise TypeError("nout must be non-negative")
 
+        for options_dict in [self._task_options_base, self._task_options_override]:
+            # Update the legacy options
+            if "cache" in options_dict:
+                cache_value = options_dict.pop("cache")
+                options_dict["cache_scope"] = CacheScope.BACKEND if cache_value else CacheScope.CSE
+
+            # Sanitize the cache options into enums
+            if "cache_scope" in options_dict:
+                options_dict["cache_scope"] = CacheScope(options_dict["cache_scope"])
+
+            if "check_valid" in options_dict:
+                options_dict["check_valid"] = CacheCheckValid(options_dict["check_valid"])
+
     @property
     def fullname(self) -> str:
         """
@@ -296,7 +338,11 @@ class Task(Value, Generic[Func]):
         return cast(
             Result,
             TaskExpression(
-                self.fullname, args, kwargs, self._task_options_override, length=self.nout
+                self.fullname,
+                args,
+                kwargs,
+                task_options=self._task_options_override,
+                length=self.nout,
             ),
         )
 
@@ -322,7 +368,8 @@ class Task(Value, Generic[Func]):
             **self._task_options_override,
             **task_options_update,
         }
-        return Task(
+        # Be sure to clone the actual type, in case it's a derived one.
+        return self.__class__(
             self.func,
             name=self.name,
             namespace=self.namespace,
@@ -334,6 +381,13 @@ class Task(Value, Generic[Func]):
             task_options_override=new_task_options_update,
         )
 
+    def update_context(self, context: dict = {}, **kwargs: Any) -> "Task[Func]":
+        """
+        Update the context variables for the task.
+        """
+        prev_context = self._task_options_override.get("_context_override", {})
+        return self.options(_context_override=merge_dicts([prev_context, context, kwargs]))
+
     def _calc_hash(self) -> str:
         """The hash is designed for checking equality of `Task`s for the purposes of caching.
         That is, results of this Task may be cached if the hash value is the same, regardless
@@ -358,10 +412,8 @@ class Task(Value, Generic[Func]):
         if self.version is None:
             if self.source:
                 source = self.source
-            elif self.func:
-                source = get_func_source(self.func)
             else:
-                source = ""
+                source = get_func_source(self.func)
             return hash_struct(
                 ["Task", self.fullname, "source", source] + hash_includes_hash + task_options_hash
             )
@@ -415,6 +467,8 @@ class Task(Value, Generic[Func]):
 
         self._signature = None
 
+        self._validate()
+
     def is_valid(self) -> bool:
         """
         Returns True if the Task Value is still valid (task hash matches registry).
@@ -446,9 +500,11 @@ class Task(Value, Generic[Func]):
         """
         Signature of the function wrapped by the task.
         """
-        assert self.func
         if not self._signature:
-            self._signature = inspect.signature(self.func)
+            if self.wrapped_task and not self.get_task_option("use_wrapper_signature", False):
+                self._signature = inspect.signature(self.inner_task.func)
+            else:
+                self._signature = inspect.signature(self.func)
         return self._signature
 
     @property
@@ -459,6 +515,35 @@ class Task(Value, Generic[Func]):
         else:
             return self.func.__module__
 
+    @property
+    def wrapped_task(self) -> Optional["Task"]:
+        """
+        Returns the wrapped task if this task is a task decorator, `None` otherwise.
+        """
+        registry = get_task_registry()
+        inner_task_name = self.get_task_option("wrapped_task")
+        if inner_task_name:
+            return registry.get(inner_task_name)
+        else:
+            return None
+
+    @property
+    def inner_task(self) -> "Task":
+        """
+        Returns the inner most wrapped task, or self if one doesn't exist.
+        """
+        registry = get_task_registry()
+        _inner_task = self
+        while True:
+            inner_task_name = _inner_task.get_task_option("wrapped_task")
+            if not inner_task_name:
+                break
+            next_task = registry.get(inner_task_name)
+            if not next_task:
+                break
+            _inner_task = next_task
+        return _inner_task
+
 
 class SchedulerTask(Task[Func]):
     """
@@ -469,7 +554,16 @@ class SchedulerTask(Task[Func]):
         """
         Returns a lazy Expression of calling the task.
         """
-        return cast(Result, SchedulerExpression(self.fullname, args, kwargs))
+        return cast(
+            Result,
+            SchedulerExpression(
+                self.fullname,
+                args,
+                kwargs,
+                task_options=self._task_options_override,
+                length=self.nout,
+            ),
+        )
 
     __call__ = cast(Func, _call)
 
@@ -509,7 +603,7 @@ class PartialTask(Task[Func], Generic[Func, Func2]):
     ) -> Result:
         # By calling the original task, we ensure that a normal pre-registered
         # task will be the one in the CallGraph recording.
-        return self.task(*self.args, *args, **self.kwargs, **kwargs)
+        return self.task(*self.args, *args, **{**self.kwargs, **kwargs})
 
     # Cast the signature to match the wrapped function.
     __call__: Func = cast(Func, _call)
@@ -526,11 +620,11 @@ class PartialTask(Task[Func], Generic[Func, Func2]):
         """
         Sets state from pickle.
         """
-        super().__setstate__(state)
         self.task = Task.__new__(Task)
         self.task.__setstate__(state["task"])
         self.args = state["args"]
         self.kwargs = state["kwargs"]
+        super().__setstate__(state)
 
     def _calc_hash(self) -> str:
         return hash_struct(
@@ -556,10 +650,38 @@ class PartialTask(Task[Func], Generic[Func, Func2]):
             self.task.options(**task_options_update).partial(*self.args, **self.kwargs),
         )
 
+    @overload
+    def get_task_option(self, option_name: str) -> Optional[Any]: ...
+
+    @overload
+    def get_task_option(self, option_name: str, default: T) -> T: ...
+
+    def get_task_option(self, option_name: str, default: Optional[T] = None) -> Optional[T]:
+        """
+        Fetch the requested option, preferring run-time updates over options from task
+        construction. Like the dictionary `get` method, returns the default
+        a `KeyError` on missing keys.
+        """
+        return self.task.get_task_option(option_name, default)
+
+    def get_task_options(self) -> dict:
+        """
+        Merge and return the task options.
+        """
+        return self.task.get_task_options()
+
+    def has_task_option(self, option_name: str) -> bool:
+        """
+        Return true if the task has an option with name `option_name`
+        """
+        return self.task.has_task_option(option_name)
+
     # Note: we can't parameterize PartialTask to a more specific type at this
     # time, due to the complexity of calculating the remaining parameter signature.
     def partial(
-        self: "PartialTask[Callable[..., Result], Callable[..., Result]]", *args, **kwargs
+        self: "PartialTask[Callable[..., Result], Callable[..., Result]]",
+        *args,
+        **kwargs,
     ) -> "PartialTask[Callable[..., Result], Callable[..., Result]]":
         """
         Partially apply some arguments to the Task.
@@ -576,8 +698,7 @@ class PartialTask(Task[Func], Generic[Func, Func2]):
 @overload
 def task(
     func: Func,
-) -> Task[Func]:
-    ...
+) -> Task[Func]: ...
 
 
 @overload
@@ -591,8 +712,7 @@ def task(
     hash_includes: Optional[list] = None,
     source: Optional[str] = None,
     **task_options_base: Any,
-) -> Callable[[Func], Task[Func]]:
-    ...
+) -> Callable[[Func], Task[Func]]: ...
 
 
 def task(
@@ -670,7 +790,10 @@ def task(
 
 
 def scheduler_task(
-    name: Optional[str] = None, namespace: Optional[str] = None, version: str = "1"
+    name: Optional[str] = None,
+    namespace: Optional[str] = None,
+    version: str = "1",
+    **task_options_base: Any,
 ) -> Callable[[Callable[..., Promise[Result]]], SchedulerTask[Callable[..., Result]]]:
     """
     Decorator to register a function as a scheduler task.
@@ -718,11 +841,17 @@ def scheduler_task(
             return cond(result, task2(), task3())
     """
 
-    def deco(func: Callable[..., Promise[Result]]) -> SchedulerTask[Callable[..., Result]]:
+    def deco(
+        func: Callable[..., Promise[Result]],
+    ) -> SchedulerTask[Callable[..., Result]]:
         nonlocal name, namespace
 
         _task: SchedulerTask[Callable[..., Result]] = SchedulerTask(
-            func, name=name, namespace=namespace, version=version
+            func,
+            name=name,
+            namespace=namespace,
+            version=version,
+            task_options_base=task_options_base,
         )
         get_task_registry().add(_task)
         return _task
@@ -733,6 +862,7 @@ def scheduler_task(
 def wraps_task(
     wrapper_name: Optional[str] = None,
     wrapper_hash_includes: list = [],
+    use_wrapper_signature: bool = False,
     **wrapper_task_options_base: Any,
 ) -> Callable[[Callable[[Task[Func]], Func]], Callable[[Func], Task[Func]]]:
     """A helper for creating new decorators that can be used like `@task`, that allow us to
@@ -815,11 +945,16 @@ def wraps_task(
         If provided, extra data that should be hashed. That is, extra data that should be
         considered as part of cache invalidation. This list may be reordered without impacting
         the computation. Each list item must be hashable by `redun.value.TypeRegistry.get_hash`
+    use_wrapper_signature : bool
+        By default (False), the resulting task will keep the parameter signature of its inner most wrapped
+        task. Set this to True, in order to use the parameter signature of the wrapper task.
+        Signatures are used to automatically infer command line options, among other things.
     """
 
-    def transform_wrapper(wrapper_func: Callable[[Task], Func]) -> Callable[[Func], Task[Func]]:
+    def transform_wrapper(
+        wrapper_func: Callable[[Task], Func],
+    ) -> Callable[[Func], Task[Func]]:
         def create_tasks(inner_func_or_task: Union[Func, Task[Func]]) -> Task[Func]:
-
             # As a convenience, create the lowest level Task on the fly.
             if isinstance(inner_func_or_task, Task):
                 hidden_inner_task = inner_func_or_task
@@ -840,15 +975,14 @@ def wraps_task(
 
                 old_fullname = task_.fullname
                 if task_.namespace != "":
-                    task_.namespace = f"{task_.namespace}.{suffix}"
+                    new_namespace = f"{task_.namespace}.{suffix}"
                 else:
-                    task_.namespace = suffix
-                # Fix the hash, which we have to do manually.
-                task_.recompute_hash()
+                    new_namespace = suffix
 
-                new_fullname = task_.fullname
-                get_task_registry().rename(old_fullname, new_fullname)
-                return new_fullname
+                new_task = get_task_registry().rename(
+                    old_fullname, new_name=task_.name, new_namespace=new_namespace
+                )
+                return new_task.fullname
 
             nonlocal wrapper_name
             if not wrapper_name:
@@ -872,6 +1006,7 @@ def wraps_task(
                 name=visible_name,
                 namespace=visible_namespace,
                 wrapped_task=hidden_inner_task.fullname,
+                use_wrapper_signature=use_wrapper_signature,
                 load_module=hidden_inner_task.load_module,
                 hash_includes=wrapper_hash_includes + wrapped_hash_data,
                 **wrapper_task_options_base,
@@ -887,15 +1022,16 @@ def wraps_task(
 class TaskRegistry:
     """
     A registry of currently registered Tasks.
-
     The @task() decorator registers tasks to the current registry.
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self._tasks: Dict[str, Task] = {}
+        self.task_hashes: set[str] = set()
 
     def add(self, task: Task) -> None:
         self._tasks[task.fullname] = task
+        self.task_hashes.add(task.hash)
 
     def get(self, task_name: Optional[str] = None, hash: Optional[str] = None) -> Optional[Task]:
         if task_name:
@@ -908,10 +1044,15 @@ class TaskRegistry:
         else:
             raise ValueError("No task field given.")
 
-    def rename(self, old_name: str, new_name: str) -> None:
+    def rename(self, old_name: str, new_namespace: str, new_name: str) -> Task:
         assert old_name in self._tasks
         task = self._tasks.pop(old_name)
-        self._tasks[new_name] = task
+        self.task_hashes.remove(task.hash)
+
+        task.namespace = new_namespace
+        task.name = new_name
+        self.add(task)
+        return task
 
     def __iter__(self) -> Iterable[Task]:
         return iter(self._tasks.values())
@@ -919,3 +1060,42 @@ class TaskRegistry:
 
 # Global singleton task registry.
 _task_registry = TaskRegistry()
+
+
+def hash_args_eval(
+    type_registry: TypeRegistry, task: Task, args: tuple, kwargs: dict
+) -> Tuple[str, str]:
+    """
+    Compute eval_hash and args_hash for a task call, filtering out config args before hashing.
+    """
+    # Filter out config args from args and kwargs.
+    sig = task.signature
+    config_args: List = task.get_task_option("config_args", [])
+
+    # Determine the variadic parameter if it exists.
+    var_param_name: typing.Optional[str] = None
+    for param in sig.parameters.values():
+        if param.kind == inspect.Parameter.VAR_POSITIONAL:
+            var_param_name = param.name
+            break
+
+    # Filter args to remove config_args.
+    args2 = [
+        arg_value
+        for arg_name, arg_value in zip(sig.parameters, args)
+        if arg_name not in config_args
+    ]
+
+    # Additional arguments are assumed to be variadic arguments.
+    args2.extend(
+        arg_value for arg_value in args[len(sig.parameters) :] if var_param_name not in config_args
+    )
+
+    # Filter kwargs.
+    kwargs2 = {
+        arg_name: arg_value
+        for arg_name, arg_value in kwargs.items()
+        if arg_name not in config_args
+    }
+
+    return hash_eval(type_registry, task.hash, args2, kwargs2)
