@@ -4,7 +4,7 @@ import pickle
 import time
 import unittest.mock
 import uuid
-from typing import cast
+from typing import Any, Dict, cast
 from unittest.mock import Mock, patch
 
 import boto3
@@ -24,9 +24,11 @@ from redun.executors.aws_batch import (
     SUCCEEDED,
     AWSBatchExecutor,
     batch_submit,
+    equiv_job_def,
     get_batch_job_name,
     get_hash_from_job_name,
     get_job_definition,
+    get_job_details,
     get_job_log_stream,
     get_or_create_job_definition,
     iter_batch_job_log_lines,
@@ -90,18 +92,22 @@ def test_get_job_definition(get_aws_client_mock) -> None:
 @patch("redun.executors.aws_batch.get_job_definition")
 def test_required_job_def_name(get_job_definition_mock, _) -> None:
     """
-    Confirm that job_def_name is required when autocreate_job is False.
+    Confirm that job_def_name is required when autocreate_job_def is False.
     """
     # A job_def_name is required when autocreate is False.
     with pytest.raises(AssertionError):
-        batch_submit({"command": ["ls"]}, "queue", "image", autocreate_job=False)
+        batch_submit({"command": ["ls"]}, "queue", "image", autocreate_job_def=False)
 
     # When the required job_def_name is supplied, an error should be raised if a matching
     # definition cannot be found.
     get_job_definition_mock.return_value = {}
     with pytest.raises(ValueError):
         batch_submit(
-            {"command": ["ls"]}, "queue", "image", job_def_name="NONEXISTENT", autocreate_job=False
+            {"command": ["ls"]},
+            "queue",
+            "image",
+            job_def_name="NONEXISTENT",
+            autocreate_job_def=False,
         )
 
 
@@ -147,13 +153,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
         containerProperties={
             "command": ["ls"],
             "image": "an-image:latest",
-            "vcpus": 1,
-            "memory": 4,
             "jobRoleArn": "aRole",
             "environment": [],
             "mountPoints": [],
             "volumes": [],
-            "resourceRequirements": [],
+            "resourceRequirements": [
+                {"type": "VCPU", "value": "1"},
+                {"type": "MEMORY", "value": "4096"},
+            ],
             "ulimits": [],
             "privileged": False,
         },
@@ -212,13 +219,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
                 "containerProperties": {
                     "command": ["ls"],
                     "image": "an-image:latest",
-                    "vcpus": 2,
-                    "memory": 4,
                     "jobRoleArn": "aRole",
                     "environment": [],
                     "mountPoints": [],
                     "volumes": [],
-                    "resourceRequirements": [],
+                    "resourceRequirements": [
+                        {"type": "VCPU", "value": "2"},
+                        {"type": "MEMORY", "value": "4096"},
+                    ],
                     "ulimits": [],
                     "privileged": False,
                 },
@@ -236,13 +244,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
         containerProperties={
             "command": ["ls"],
             "image": "an-image:latest",
-            "vcpus": 1,
-            "memory": 4,
             "jobRoleArn": "aRole",
             "environment": [],
             "mountPoints": [],
             "volumes": [],
-            "resourceRequirements": [],
+            "resourceRequirements": [
+                {"type": "VCPU", "value": "1"},
+                {"type": "MEMORY", "value": "4096"},
+            ],
             "ulimits": [],
             "privileged": False,
         },
@@ -257,13 +266,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
                 "containerProperties": {
                     "command": ["ls"],
                     "image": "an-image:latest",
-                    "vcpus": 2,
-                    "memory": 4,
                     "jobRoleArn": "aRole",
                     "environment": [],
                     "mountPoints": [],
                     "volumes": [],
-                    "resourceRequirements": [],
+                    "resourceRequirements": [
+                        {"type": "VCPU", "value": "1"},
+                        {"type": "MEMORY", "value": "4096"},
+                    ],
                     "ulimits": [],
                     "privileged": False,
                 },
@@ -285,13 +295,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
                     "container": {
                         "command": ["ls"],
                         "image": "an-image:latest",
-                        "vcpus": 1,
-                        "memory": 4,
                         "jobRoleArn": "aRole",
                         "environment": [],
                         "mountPoints": [],
                         "volumes": [],
-                        "resourceRequirements": [],
+                        "resourceRequirements": [
+                            {"type": "VCPU", "value": "1"},
+                            {"type": "MEMORY", "value": "4096"},
+                        ],
                         "ulimits": [
                             {
                                 "name": "nofile",
@@ -307,13 +318,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
                     "container": {
                         "command": ["ls"],
                         "image": "an-image:latest",
-                        "vcpus": 1,
-                        "memory": 4,
                         "jobRoleArn": "aRole",
                         "environment": [],
                         "mountPoints": [],
                         "volumes": [],
-                        "resourceRequirements": [],
+                        "resourceRequirements": [
+                            {"type": "VCPU", "value": "1"},
+                            {"type": "MEMORY", "value": "4096"},
+                        ],
                         "ulimits": [
                             {
                                 "name": "nofile",
@@ -352,13 +364,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
                 "containerProperties": {
                     "command": ["ls"],
                     "image": "an-image:latest",
-                    "vcpus": 2,
-                    "memory": 4,
                     "jobRoleArn": "aRole",
                     "environment": [],
                     "mountPoints": [],
                     "volumes": [],
-                    "resourceRequirements": [],
+                    "resourceRequirements": [
+                        {"type": "VCPU", "value": "2"},
+                        {"type": "MEMORY", "value": "4096"},
+                    ],
                     "ulimits": [],
                     "privileged": False,
                 },
@@ -376,13 +389,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
         containerProperties={
             "command": ["ls"],
             "image": "an-image:custom",
-            "vcpus": 1,
-            "memory": 4,
             "jobRoleArn": "aRole",
             "environment": [],
             "mountPoints": [],
             "volumes": [],
-            "resourceRequirements": [],
+            "resourceRequirements": [
+                {"type": "VCPU", "value": "1"},
+                {"type": "MEMORY", "value": "4096"},
+            ],
             "ulimits": [],
             "privileged": False,
         },
@@ -404,7 +418,10 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
     describe_job_definitions.return_value = {"jobDefinitions": [job_def]}
     register_job_definition.reset_mock()
     _get_or_create_job_definition(
-        job_def_name="test-jd", image="an-image:latest", role="aRole", job_def_extra=ulimit_option
+        job_def_name="test-jd",
+        image="an-image:latest",
+        role="aRole",
+        job_def_extra=ulimit_option,
     )
     register_job_definition.assert_called_with(
         jobDefinitionName="test-jd",
@@ -412,13 +429,14 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
         containerProperties={
             "command": ["ls"],
             "image": "an-image:latest",
-            "vcpus": 1,
-            "memory": 4,
             "jobRoleArn": "aRole",
             "environment": [],
             "mountPoints": [],
             "volumes": [],
-            "resourceRequirements": [],
+            "resourceRequirements": [
+                {"type": "VCPU", "value": "1"},
+                {"type": "MEMORY", "value": "4096"},
+            ],
             "ulimits": [
                 {
                     "name": "nofile",
@@ -431,6 +449,25 @@ def test_get_or_create_job_definition(get_aws_client_mock) -> None:
     )
 
 
+@patch("redun.executors.aws_utils.get_aws_client")
+def test_job_definition_role(get_aws_client_mock) -> None:
+    """
+    Confirm that jobs roles are set correctly.
+    """
+
+    # The default role is the AWS account's ecsTaskExecutionRole.
+    details = get_job_details("image")
+    assert details["containerProperties"]["jobRoleArn"].endswith("role/ecsTaskExecutionRole")
+
+    # If specified, use the user's role.
+    details = get_job_details("image", role="aRole")
+    assert details["containerProperties"]["jobRoleArn"] == "aRole"
+
+    # If the "none" role is specified, do not set the role.
+    details = get_job_details("image", role="none")
+    assert "jobRoleArn" not in details["containerProperties"]
+
+
 @patch("redun.executors.aws_utils.get_aws_client")
 @patch("redun.executors.aws_batch.get_or_create_job_definition")
 def test_job_submit_resources(get_or_create_job_definition_mock, get_aws_client_mock) -> None:
@@ -442,7 +479,10 @@ def test_job_submit_resources(get_or_create_job_definition_mock, get_aws_client_
 
     # 1. Single node
     batch_submit(
-        batch_job_args={"containerOverrides": {"command": ["test_command"]}, "user": "user-test"},
+        batch_job_args={
+            "containerOverrides": {"command": ["test_command"]},
+            "user": "user-test",
+        },
         job_name="test",
         queue="test_queue",
         image="test_image",
@@ -459,9 +499,11 @@ def test_job_submit_resources(get_or_create_job_definition_mock, get_aws_client_
         retryStrategy={"attempts": 1},
         containerOverrides={
             "command": ["test_command"],
-            "vcpus": 4,
-            "memory": 8192,
-            "resourceRequirements": [{"type": "GPU", "value": "2"}],
+            "resourceRequirements": [
+                {"type": "VCPU", "value": "4"},
+                {"type": "MEMORY", "value": "8192"},
+                {"type": "GPU", "value": "2"},
+            ],
         },
         propagateTags=True,
         user="user-test",
@@ -504,18 +546,22 @@ def test_job_submit_resources(get_or_create_job_definition_mock, get_aws_client_
                     "targetNodes": "0",
                     "containerOverrides": {
                         "command": ["command_rank_0"],
-                        "vcpus": 4,
-                        "memory": 8192,
-                        "resourceRequirements": [{"type": "GPU", "value": "2"}],
+                        "resourceRequirements": [
+                            {"type": "VCPU", "value": "4"},
+                            {"type": "MEMORY", "value": "8192"},
+                            {"type": "GPU", "value": "2"},
+                        ],
                     },
                 },
                 {
                     "targetNodes": "1:",
                     "containerOverrides": {
                         "command": ["command_rank_1"],
-                        "vcpus": 4,
-                        "memory": 8192,
-                        "resourceRequirements": [{"type": "GPU", "value": "2"}],
+                        "resourceRequirements": [
+                            {"type": "VCPU", "value": "4"},
+                            {"type": "MEMORY", "value": "8192"},
+                            {"type": "GPU", "value": "2"},
+                        ],
                     },
                 },
             ]
@@ -547,8 +593,7 @@ def test_batch_tags(scheduler: Scheduler) -> None:
         return x
 
     exec1 = Execution("123")
-    job = Job(task1(10), execution=exec1)
-    job.task = task1
+    job = Job(task1, task1(10), execution=exec1)
 
     batch_tags = executor._get_job_options(job)["batch_tags"]
     assert batch_tags == {
@@ -586,8 +631,7 @@ def test_batch_tags_no_default(scheduler: Scheduler) -> None:
         return x
 
     exec1 = Execution("123")
-    job = Job(task1(10), execution=exec1)
-    job.task = task1
+    job = Job(task1, task1(10), execution=exec1)
 
     batch_tags = executor._get_job_options(job)["batch_tags"]
     assert batch_tags == {
@@ -665,7 +709,7 @@ def task1(x):
     """
     )
 
-    job = Job(a_task())
+    job = Job(a_task, a_task())
     job.id = job_id
     job.eval_hash = "eval_hash"
     code_file = package_code(s3_scratch_prefix)
@@ -751,7 +795,7 @@ def task1(x):
     """
     )
 
-    job = Job(a_task())
+    job = Job(a_task, a_task())
     job.id = job_id
     job.eval_hash = "eval_hash"
     code_file = File("s3://example-bucket/redun/code/123.tar.gz")
@@ -826,7 +870,7 @@ def task1(x):
     """
     )
 
-    job = Job(task1())
+    job = Job(task1, task1())
     job.id = job_id
     job.eval_hash = "eval_hash"
     code_file = package_code(s3_scratch_prefix)
@@ -940,7 +984,7 @@ def task1(x):
 
     module = import_script("path/to/workflow.py")
 
-    job = Job(module.task1())
+    job = Job(module.task1, module.task1())
     job.id = job_id
     job.eval_hash = "eval_hash"
     code_file = package_code(s3_scratch_prefix)
@@ -1014,8 +1058,7 @@ def test_parse_job_error() -> None:
         return "echo hello!"
 
     expr = task1(10)
-    job = Job(expr)
-    job.task = task1
+    job = Job(task1, expr)
     job.eval_hash = "eval_hash"
 
     # Normal task, no error file.
@@ -1035,8 +1078,7 @@ def test_parse_job_error() -> None:
 
     # Create a script task and job.
     expr2 = task_script1()
-    job2 = Job(expr2)
-    job2.task = task_script1
+    job2 = Job(task_script1, expr2)
     job2.eval_hash = "eval_hash2"
 
     # Script task without an error file should retutn a generic error.
@@ -1159,7 +1201,9 @@ def test_iter_batch_job_logs(aws_describe_jobs_mock) -> None:
     assert list(iter_batch_job_logs(job_id, required=False)) == []
 
 
-def mock_executor(scheduler, debug=False, code_package=False):
+def mock_executor(
+    scheduler, custom_config_args: Dict[str, Any] = {}, debug=False, code_package=False
+):
     """
     Returns an AWSBatchExecutor with AWS API mocks.
     """
@@ -1178,15 +1222,16 @@ def mock_executor(scheduler, debug=False, code_package=False):
                 "job_stale_time": 0.01,
                 "code_package": code_package,
                 "debug": debug,
+                **custom_config_args,
             }
         }
     )
     executor = AWSBatchExecutor("batch", scheduler, config["batch"])
 
-    executor.get_jobs = Mock()
+    executor.get_jobs = Mock()  # type:ignore[method-assign]
     executor.get_jobs.return_value = []
 
-    executor.get_array_child_jobs = Mock()
+    executor.get_array_child_jobs = Mock()  # type:ignore[method-assign]
     executor.get_array_child_jobs.return_value = []
 
     s3_client = boto3.client("s3", region_name="us-east-1")
@@ -1201,7 +1246,10 @@ def mock_executor(scheduler, debug=False, code_package=False):
 @patch("redun.executors.aws_batch.iter_batch_job_status")
 @patch("redun.executors.aws_batch.batch_submit")
 def test_executor(
-    batch_submit_mock, iter_batch_job_status_mock, parse_job_logs_mock, get_aws_user_mock
+    batch_submit_mock,
+    iter_batch_job_status_mock,
+    parse_job_logs_mock,
+    get_aws_user_mock,
 ) -> None:
     """
     Ensure that we can submit job to AWSBatchExecutor.
@@ -1214,7 +1262,7 @@ def test_executor(
     parse_job_logs_mock.return_value = []
 
     scheduler = mock_scheduler()
-    executor = mock_executor(scheduler)
+    executor = mock_executor(scheduler, {"timeout": 5, "shared_memory": 5, "privileged": True})
     executor.start()
 
     try:
@@ -1224,10 +1272,10 @@ def test_executor(
 
         # Submit redun job that will succeed.
         expr = task1(10)
-        job = Job(expr)
-        job.task = task1
+        job = Job(task1, expr)
         job.eval_hash = "eval_hash"
-        executor.submit(job, [10], {})
+        job.args = ((10,), {})
+        executor.submit(job)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1242,7 +1290,11 @@ def test_executor(
             "vcpus": 1,
             "gpus": 0,
             "memory": 4,
-            "shared_memory": None,
+            "shared_memory": 5,
+            "timeout": 5,
+            "privileged": True,
+            "autocreate_job_def": True,
+            "job_def_name": None,
             "role": None,
             "retries": 1,
             "aws_region": "us-west-2",
@@ -1255,6 +1307,8 @@ def test_executor(
                 "redun_project": "",
                 "redun_task_name": "task1",
             },
+            "share_id": None,
+            "scheduling_priority_override": None,
         }
 
         batch_submit_mock.return_value = {
@@ -1263,10 +1317,10 @@ def test_executor(
 
         # Submit redun job that will fail.
         expr2 = task1.options(memory=8)("a")
-        job2 = Job(expr2)
-        job2.task = task1
+        job2 = Job(task1, expr2)
         job2.eval_hash = "eval_hash2"
-        executor.submit(job2, ["a"], {})
+        job2.args = (("a",), {})
+        executor.submit(job2)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1280,7 +1334,11 @@ def test_executor(
             "vcpus": 1,
             "gpus": 0,
             "memory": 8,
-            "shared_memory": None,
+            "shared_memory": 5,
+            "timeout": 5,
+            "privileged": True,
+            "autocreate_job_def": True,
+            "job_def_name": None,
             "role": None,
             "retries": 1,
             "aws_region": "us-west-2",
@@ -1293,6 +1351,8 @@ def test_executor(
                 "redun_project": "",
                 "redun_task_name": "task1",
             },
+            "share_id": None,
+            "scheduling_priority_override": None,
         }
 
         # Simulate AWS Batch completing job.
@@ -1311,7 +1371,11 @@ def test_executor(
                     "status": SUCCEEDED,
                     "container": {"logStreamName": "log1"},
                 },
-                {"jobId": batch_job2_id, "status": FAILED, "container": {"logStreamName": "log2"}},
+                {
+                    "jobId": batch_job2_id,
+                    "status": FAILED,
+                    "container": {"logStreamName": "log2"},
+                },
             ]
         )
 
@@ -1336,7 +1400,10 @@ def test_executor(
 @patch("redun.executors.aws_batch.iter_batch_job_status")
 @patch("redun.executors.aws_batch.batch_submit")
 def test_executor_multinode(
-    batch_submit_mock, iter_batch_job_status_mock, parse_job_logs_mock, get_aws_user_mock
+    batch_submit_mock,
+    iter_batch_job_status_mock,
+    parse_job_logs_mock,
+    get_aws_user_mock,
 ) -> None:
     """
     Ensure that we can submit job to AWSBatchExecutor.
@@ -1358,10 +1425,10 @@ def test_executor_multinode(
 
         # Submit redun job that will succeed.
         expr = task1.options(num_nodes=4)(10)
-        job = Job(expr)
-        job.task = task1
+        job = Job(task1, expr)
         job.eval_hash = "eval_hash"
-        executor.submit(job, [10], {})
+        job.args = ((10,), {})
+        executor.submit(job)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1377,6 +1444,10 @@ def test_executor_multinode(
             "gpus": 0,
             "memory": 4,
             "shared_memory": None,
+            "timeout": None,
+            "privileged": False,
+            "autocreate_job_def": True,
+            "job_def_name": None,
             "role": None,
             "retries": 1,
             "aws_region": "us-west-2",
@@ -1389,8 +1460,13 @@ def test_executor_multinode(
                 "redun_project": "",
                 "redun_task_name": "task1",
             },
+            "share_id": None,
+            "scheduling_priority_override": None,
         }
-        job.job_tags == [("aws_batch_job", "batch-job-id#0"), ("aws_log_stream", "log1")]
+        job.job_tags == [
+            ("aws_batch_job", "batch-job-id#0"),
+            ("aws_log_stream", "log1"),
+        ]
     finally:
         # Ensure executor threads stop before continuing.
         executor.stop()
@@ -1425,10 +1501,10 @@ def test_executor_docker(
 
         # Submit redun job that will succeed.
         expr = task1(10)
-        job = Job(expr)
-        job.task = task1
+        job = Job(task1, expr)
         job.eval_hash = "eval_hash"
-        executor.submit(job, [10], {})
+        job.args = ((10,), {})
+        executor.submit(job)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1443,6 +1519,7 @@ def test_executor_docker(
             "vcpus": 1,
             "volumes": [(scratch_dir, scratch_dir)],
             "interactive": False,
+            "include_aws_env": True,
         }
 
         run_docker_mock.reset_mock()
@@ -1450,10 +1527,10 @@ def test_executor_docker(
 
         # Hand create Job and submit.
         expr2 = task1("a")
-        job2 = Job(expr2)
-        job2.task = task1
+        job2 = Job(task1, expr2)
         job2.eval_hash = "eval_hash2"
-        executor.submit(job2, ["a"], {})
+        job2.args = (("a",), {})
+        executor.submit(job2)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1467,6 +1544,7 @@ def test_executor_docker(
             "vcpus": 1,
             "volumes": [(scratch_dir, scratch_dir)],
             "interactive": False,
+            "include_aws_env": True,
         }
 
         # Simulate output file created by job.
@@ -1524,10 +1602,10 @@ def test_executor_job_debug(
 
         # Submit redun job that will succeed.
         expr = task1.options(debug=True)(10)
-        job = Job(expr)
-        job.task = task1
+        job = Job(task1, expr)
         job.eval_hash = "eval_hash"
-        executor.submit(job, [10], {})
+        job.args = ((10,), {})
+        executor.submit(job)
 
         # Let job get stale so job arrayer actually submits it.
         wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1542,6 +1620,7 @@ def test_executor_job_debug(
             "vcpus": 1,
             "volumes": [(scratch_dir, scratch_dir)],
             "interactive": False,
+            "include_aws_env": True,
         }
 
         # Simulate output file created by job.
@@ -1570,7 +1649,10 @@ def test_executor_job_debug(
 @patch("redun.executors.aws_batch.iter_batch_job_status")
 @patch("redun.executors.aws_batch.batch_submit")
 def test_executor_error_override(
-    batch_submit_mock, iter_batch_job_status_mock, parse_job_logs_mock, get_aws_user_mock
+    batch_submit_mock: Mock,
+    iter_batch_job_status_mock: Mock,
+    parse_job_logs_mock: Mock,
+    get_aws_user_mock: Mock,
 ) -> None:
     """
     Some AWS Batch errors should be overridden.
@@ -1601,10 +1683,10 @@ def test_executor_error_override(
 
     # Submit redun job that will succeed at the redun-level.
     expr = task1.options(memory=8)("a")
-    job = Job(expr)
-    job.task = task1
+    job = Job(task1, expr)
     job.eval_hash = "eval_hash"
-    executor.submit(job, ["a"], {})
+    job.args = (("a",), {})
+    executor.submit(job)
 
     # Let job get stale so job arrayer actually submits it.
     wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1615,10 +1697,10 @@ def test_executor_error_override(
 
     # Submit redun job that will succeed at the redun-level.
     expr_script = task_script1.options(memory=8)("a")
-    job_script = Job(expr_script)
-    job_script.task = task_script1
+    job_script = Job(task_script1, expr_script)
     job_script.eval_hash = "eval_script_hash"
-    executor.submit(job_script, ["a"], {})
+    job_script.args = (("a",), {})
+    executor.submit(job_script)
 
     # Let job get stale so job arrayer actually submits it.
     wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1710,10 +1792,10 @@ def test_interactive(run_docker_mock, iter_local_job_status_mock, get_aws_user_m
 
     # Hand create Job and submit.
     expr = task1.options(interactive=True)(10)
-    job = Job(expr)
-    job.task = task1
+    job = Job(task1, expr)
     job.eval_hash = "eval_hash"
-    executor.submit(job, [10], {})
+    job.args = ((10,), {})
+    executor.submit(job)
 
     # Let job get stale so job arrayer actually submits it
     wait_until(lambda: executor.arrayer.num_pending == 0)
@@ -1728,6 +1810,7 @@ def test_interactive(run_docker_mock, iter_local_job_status_mock, get_aws_user_m
         "shared_memory": None,
         "vcpus": 1,
         "volumes": [(scratch_dir, scratch_dir)],
+        "include_aws_env": True,
     }
 
     # Cleanly stop executor.
@@ -1804,7 +1887,10 @@ def test_executor_inflight_array_job() -> None:
 
     # Check query for child job happened.
     assert executor.get_array_child_jobs.call_args
-    assert executor.get_array_child_jobs.call_args[0] == ("carrots", BATCH_JOB_STATUSES.inflight)
+    assert executor.get_array_child_jobs.call_args[0] == (
+        "carrots",
+        BATCH_JOB_STATUSES.inflight,
+    )
 
     # Make sure child jobs (and not parent) ended up in pending batch jobs.
     assert executor.preexisting_batch_jobs == {
@@ -1836,24 +1922,25 @@ def test_code_packaging(
     assert package_code_mock.call_count == 0
 
     # Hand create jobs.
-    job1 = Job(task1(10))
+    job1 = Job(task1, task1(10))
     job1.id = "1"
     job1.task = task1
     job1.eval_hash = "eval_hash"
+    job1.args = ((10,), {})
 
-    job2 = Job(task1(20))
+    job2 = Job(task1, task1(20))
     job2.id = "2"
-    job2.task = task1
     job2.eval_hash = "eval_hash"
+    job2.args = ((20,), {})
 
     # Submit a job and ensure that the code was packaged.
-    executor.submit(job1, [10], {})
+    executor.submit(job1)
     assert executor.code_file
     assert executor.code_file.path == "s3://fake-bucket/code.tar.gz"
     assert package_code_mock.call_count == 1
 
     # Submit another job and ensure that code was not packaged again.
-    executor.submit(job2, [20], {})
+    executor.submit(job2)
     assert package_code_mock.call_count == 1
 
     executor.stop()
@@ -1872,24 +1959,24 @@ def test_inflight_join_only_on_first_submission(aws_describe_jobs_mock, get_aws_
     executor.start()
 
     # Hand create jobs.
-    job1 = Job(task1(10))
+    job1 = Job(task1, task1(10))
     job1.id = "1"
-    job1.task = task1
     job1.eval_hash = "eval_hash"
+    job1.args = ((10,), {})
 
-    job2 = Job(task1(20))
+    job2 = Job(task1, task1(20))
     job2.id = "2"
-    job2.task = task1
     job2.eval_hash = "eval_hash"
+    job2.args = ((20,), {})
 
     # Submit redun job.
-    executor.submit(job1, [10], {})
+    executor.submit(job1)
 
     # Ensure that inflight jobs were gathered.
     assert executor.get_jobs.call_count == 1
 
     # Submit the second job and confirm that job reuniting was not done again.
-    executor.submit(job2, [20], {})
+    executor.submit(job2)
     assert executor.get_jobs.call_count == 1
 
     executor.stop()
@@ -1927,13 +2014,13 @@ def test_executor_inflight_job(
     executor.start()
 
     # Hand create job.
-    job = Job(task1(10))
+    job = Job(task1, task1(10))
     job.id = "123"
-    job.task = task1
     job.eval_hash = "eval_hash"
+    job.args = ((10,), {})
 
     # Submit redun job.
-    executor.submit(job, [10], {})
+    executor.submit(job)
 
     # Ensure no batch jobs were submitted.
     assert batch_submit_mock.call_count == 0
@@ -1967,13 +2054,10 @@ def other_task(x, y):
 
 
 # Tests begin here
-def test_job_descrs():
+def test_job_descrs() -> None:
     """Tests the JobDescription class used to determine if Jobs are equivalent"""
-    j1 = Job(array_task(1))
-    j1.task = array_task
-
-    j2 = Job(array_task(2))
-    j2.task = array_task
+    j1 = Job(array_task, array_task(1))
+    j2 = Job(array_task, array_task(2))
 
     a = job_array.JobDescription(j1)
     b = job_array.JobDescription(j2)
@@ -1981,45 +2065,52 @@ def test_job_descrs():
     assert hash(a) == hash(b)
     assert a == b
 
-    # JobDescription should validate that Job has a task set.
-    j3 = Job(other_task(1, y=2))
-    with pytest.raises(AssertionError):
-        c = job_array.JobDescription(j3)
-    j3.task = other_task
+    j3 = Job(other_task, other_task(1, y=2))
     c = job_array.JobDescription(j3)
-
     assert a != c
 
 
 @mock_s3
-def test_job_staleness():
+def test_job_staleness() -> None:
     """Tests staleness criteria for array'ing jobs"""
-    j1 = Job(array_task(1))
-    j1.task = array_task
+    j1 = Job(array_task, array_task(1))
+    j1.args = ((1,), {})
     d = job_array.JobDescription(j1)
 
     sched = mock_scheduler()
     exec = mock_executor(sched)
-    arr = job_array.JobArrayer(exec, submit_interval=10000.0, stale_time=0.05, min_array_size=5)
+    arr = job_array.JobArrayer(
+        exec._submit_jobs,
+        exec._on_error,
+        submit_interval=10000.0,
+        stale_time=0.05,
+        min_array_size=5,
+    )
 
     for i in range(10):
-        arr.add_job(j1, args=(i), kwargs={})
+        arr.add_job(j1)
 
     assert arr.get_stale_descrs() == []
     wait_until(lambda: arr.get_stale_descrs() == [d])
 
 
 @mock_s3
-def test_arrayer_thread():
+def test_arrayer_thread() -> None:
     """Tests that the arrayer monitor thread can be restarted after exit"""
-    j1 = Job(array_task(1))
-    j1.task = array_task
+    j1 = Job(array_task, array_task(1))
+    j1.args = ((1,), {})
 
     sched = mock_scheduler()
     exec = mock_executor(sched)
-    arr = job_array.JobArrayer(exec, submit_interval=10000.0, stale_time=0.05, min_array_size=5)
+    arr = job_array.JobArrayer(
+        exec._submit_jobs,
+        exec._on_error,
+        submit_interval=10000.0,
+        stale_time=0.05,
+        min_array_size=5,
+    )
 
-    arr.add_job(j1, args=(1), kwargs={})
+    arr.add_job(j1)
     assert arr._monitor_thread.is_alive()
 
     # Stop the monitoring thread.
@@ -2027,7 +2118,7 @@ def test_arrayer_thread():
     assert not arr._monitor_thread.is_alive()
 
     # Submitting an additional job should restart the thread.
-    arr.add_job(j1, args=(2), kwargs={})
+    arr.add_job(j1)
     assert arr._monitor_thread.is_alive()
 
     arr.stop()
@@ -2037,7 +2128,9 @@ def test_arrayer_thread():
 @patch("redun.executors.aws_utils.get_aws_user", return_value="alice")
 @patch("redun.executors.aws_batch.aws_describe_jobs")
 @patch("redun.executors.aws_batch.submit_task")
-def test_jobs_are_arrayed(submit_task_mock, aws_describe_jobs_mock, get_aws_user_mock):
+def test_jobs_are_arrayed(
+    submit_task_mock: Mock, aws_describe_jobs_mock: Mock, get_aws_user_mock: Mock
+) -> None:
     """
     Tests repeated jobs are submitted as a single array job. Checks that
     job ID for the array job and child jobs end up tracked
@@ -2048,7 +2141,7 @@ def test_jobs_are_arrayed(submit_task_mock, aws_describe_jobs_mock, get_aws_user
     executor.arrayer.max_array_size = 7
 
     aws_describe_jobs_mock.return_value = iter([])
-    redun.executors.aws_batch.submit_task.side_effect = [
+    submit_task_mock.side_effect = [
         {"jobId": "first-array-job", "arrayProperties": {"size": 7}},
         {"jobId": "second-array-job", "arrayProperties": {"size": 3}},
         {"jobId": "single-job"},
@@ -2056,12 +2149,12 @@ def test_jobs_are_arrayed(submit_task_mock, aws_describe_jobs_mock, get_aws_user
 
     test_jobs = []
     for i in range(10):
-        job = Job(array_task(i))
+        job = Job(array_task, array_task(i))
         job.id = f"task_{i}"
-        job.task = array_task
         job.eval_hash = f"eval_hash_{i}"
+        job.args = ((i,), {})
 
-        executor.submit(job, (i), {})
+        executor.submit(job)
         test_jobs.append(job)
 
     # Wait for jobs to get submitted from arrayer to executor.
@@ -2083,11 +2176,11 @@ def test_jobs_are_arrayed(submit_task_mock, aws_describe_jobs_mock, get_aws_user
     assert submit_task_mock.call_count == 2
 
     # Submit a different kind of job now.
-    j = Job(other_task(3, 5))
+    j = Job(other_task, other_task(3, 5))
     j.id = "other_task"
-    j.task = other_task
     j.eval_hash = "hashbrowns"
-    executor.submit(j, (3, 5), {})
+    j.args = ((3, 5), {})
+    executor.submit(j)
 
     assert len(executor.arrayer.pending) == 1
     pending_correct["single-job"] = j
@@ -2124,15 +2217,15 @@ def test_array_disabling(submit_single_mock, get_aws_user_mock):
     executor.get_jobs.return_value = []
 
     # Submit one test job.
-    job = Job(other_task(5, 3))
+    job = Job(other_task, other_task(5, 3))
     job.id = "carrots"
-    job.task = other_task
     job.eval_hash = "why do i always say carrots in test cases idk"
-    executor.submit(job, [5, 3], {})
+    job.args = ((5, 3), {})
+    executor.submit(job)
 
     # Job should be submitted immediately.
     assert submit_single_mock.call_args
-    assert submit_single_mock.call_args[0] == (job, [5, 3], {})
+    assert submit_single_mock.call_args[0] == (job,)
 
     # Monitor thread should not run.
     assert not executor.arrayer._monitor_thread.is_alive()
@@ -2158,14 +2251,13 @@ def test_array_job_s3_setup(batch_submit_mock: Mock) -> None:
 
     test_jobs = []
     for i in range(10):
-        job = Job(other_task(i, y=2 * i))
+        job = Job(other_task, other_task(i, y=2 * i))
         job.id = f"task_{i}"
-        job.task = other_task
         job.eval_hash = f"hash_{i}"
+        job.args = ((i,), {"y": 2 * i})
         test_jobs.append(job)
 
-    pending_jobs = [job_array.PendingJob(test_jobs[i], (i,), {"y": 2 * i}) for i in range(10)]
-    array_uuid = executor.arrayer.submit_array_job(pending_jobs)
+    array_uuid = executor._submit_array_job(test_jobs)
 
     # Check input file is on S3 and contains list of (args, kwargs) tuples
     input_file = File(
@@ -2247,14 +2339,13 @@ def other_task(x, y):
 
     test_jobs = []
     for i in range(3):
-        job = Job(other_task(i, y=2 * i))
+        job = Job(other_task, other_task(i, y=2 * i))
         job.id = f"task_{i}"
-        job.task = other_task
         job.eval_hash = f"hash_{i}"
+        job.args = ((i,), {"y": 2 * i})
         test_jobs.append(job)
 
-    pending_jobs = [job_array.PendingJob(test_jobs[i], (i,), {"y": 2 * i}) for i in range(3)]
-    array_uuid = executor.arrayer.submit_array_job(pending_jobs)
+    array_uuid = executor._submit_array_job(test_jobs)
 
     # Now run 2 of those jobs and make sure they work ok
     client = RedunClient()
@@ -2370,3 +2461,108 @@ def test_get_log_stream(aws_describe_jobs_mock) -> None:
     assert aws_describe_jobs_mock.call_args == unittest.mock.call(
         ["multi_node_id#0"], aws_region=aws_region
     )
+
+
+def test_equiv_job_def() -> None:
+    """Check our equality testing for job definitions."""
+
+    basic_job = get_job_details(image="some_image", role="some_role")
+    # This is derived from a real query for a job description from batch, lightly redacted.
+    # It has lots of keys we don't set, but shouldn't cause errors.
+    existing_job = {
+        "jobDefinitionName": "jobDefinitionName",
+        "jobDefinitionArn": "jobDefinitionArn",
+        "revision": 12345,
+        "status": "ACTIVE",
+        "type": "container",
+        "parameters": {},
+        "containerProperties": {
+            "image": "some_image",
+            "vcpus": 1,
+            "memory": 4,
+            "command": ["ls"],
+            "jobRoleArn": "some_role",
+            "volumes": [],
+            "environment": [],
+            "mountPoints": [],
+            "privileged": False,
+            "ulimits": [],
+            "resourceRequirements": [],
+            "secrets": [],
+        },
+        "tags": {},
+    }
+
+    assert equiv_job_def(basic_job, existing_job)
+
+
+################################################################################################
+# Fair Share Scheduling related tests
+################################################################################################
+
+
+def test_fss_executor_config(scheduler: Scheduler) -> None:
+    """
+    FSS config variables flow are recognized by Executor.
+    """
+    # Setup executor.
+    config = Config(
+        {
+            "batch": {
+                "image": "image",
+                "queue": "queue",
+                "s3_scratch": "s3_scratch_prefix",
+                "share_id": "team1",
+                "scheduling_priority_override": 20,
+            }
+        }
+    )
+    executor = AWSBatchExecutor("batch", scheduler, config["batch"])
+
+    assert executor.image == "image"
+    assert executor.queue == "queue"
+    assert executor.s3_scratch_prefix == "s3_scratch_prefix"
+    assert isinstance(executor.code_package, dict)
+    assert executor.debug is False
+
+    @task(namespace="test")
+    def task1(x):
+        return x
+
+    exec1 = Execution("123")
+    job = Job(task1, task1(10), execution=exec1)
+
+    options = executor._get_job_options(job)
+    assert options["share_id"] == "team1"
+    assert options["scheduling_priority_override"] == 20
+
+
+def test_fss_task_override(scheduler: Scheduler) -> None:
+    """
+    FSS config variables can be overridden on Task
+    """
+    # Setup executor.
+    config = Config(
+        {
+            "batch": {
+                "image": "image",
+                "queue": "queue",
+                "s3_scratch": "s3_scratch_prefix",
+                "share_id": "team1",
+                "scheduling_priority_override": 20,
+            }
+        }
+    )
+    executor = AWSBatchExecutor("batch", scheduler, config["batch"])
+    executor._aws_user = "alice"
+
+    @task(share_id="subteam2", scheduling_priority_override=99, namespace="test")
+    def task1(x):
+        return x
+
+    exec1 = Execution("123")
+    job = Job(task1, task1(10), execution=exec1)
+
+    options = executor._get_job_options(job)
+    assert options["share_id"] == "subteam2"
+    assert options["scheduling_priority_override"] == 99
