@@ -12,7 +12,11 @@ from moto.glue.responses import GlueResponse
 
 from redun import File, task
 from redun.config import Config
-from redun.executors.aws_glue import AWSGlueExecutor, get_redun_lib_files, package_redun_lib
+from redun.executors.aws_glue import (
+    AWSGlueExecutor,
+    get_redun_lib_files,
+    package_redun_lib,
+)
 from redun.executors.aws_utils import get_aws_client
 from redun.executors.code_packaging import create_zip
 from redun.scheduler import Job
@@ -33,7 +37,7 @@ class RedunGlueBackend(moto.glue.models.GlueBackend):
 
     from moto.core import BaseBackend, BaseModel
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.job_id_num = 0
         self.job_defs: Dict[str, dict] = {}  # Job definitions.
         super(RedunGlueBackend, self).__init__()
@@ -196,6 +200,7 @@ def test_executor_config(scheduler):
                 "glue_job_name": "test_glue_job",
                 "glue_code_location": "s3://example-bucket/",
                 "redun_zip_path": "s3://example-bucket/redun.zip",
+                "glue_version": "3.0",
             }
         }
     )
@@ -212,6 +217,7 @@ def test_executor_config(scheduler):
     assert executor.interval == 1.0
     assert executor.code_package["includes"] == ["*.txt"]
     assert executor.debug is False
+    assert executor.glue_version == "3.0"
 
 
 @use_tempdir
@@ -257,10 +263,20 @@ def test_executor_inflight_glue_job(submit_job_mock, describe_jobs_mock) -> None
 
     # Set up mocks to indicate a job is inflight.
     executor.get_jobs.return_value = [
-        {"Id": "carrots", "Arguments": {"--job-hash": "hashbrowns"}, "JobRunState": "RUNNING"}
+        {
+            "Id": "carrots",
+            "Arguments": {"--job-hash": "hashbrowns"},
+            "JobRunState": "RUNNING",
+        }
     ]
     describe_jobs_mock.return_value = iter(
-        [{"Id": "carrots", "Arguments": {"--job-hash": "hashbrowns"}, "JobRunState": "RUNNING"}]
+        [
+            {
+                "Id": "carrots",
+                "Arguments": {"--job-hash": "hashbrowns"},
+                "JobRunState": "RUNNING",
+            }
+        ]
     )
 
     # Force scheduler to gather inflight jobs.
@@ -269,11 +285,11 @@ def test_executor_inflight_glue_job(submit_job_mock, describe_jobs_mock) -> None
     assert executor.preexisting_glue_jobs == {"hashbrowns": "carrots"}
 
     # Submit a job with same hash and ensure it isn't actually submitted
-    rjob = Job(task1(10))
+    rjob = Job(task1, task1(10))
     rjob.id = "123"
-    rjob.task = task1
     rjob.eval_hash = "hashbrowns"
-    executor.submit(rjob, [10], {})
+    rjob.args = ((10,), {})
+    executor.submit(rjob)
     executor.stop()
     executor._monitor_thread.join()
     executor._submit_thread.join()
@@ -306,7 +322,8 @@ def test_job_def_creation() -> None:
     assert retval["Job"]["Name"] == executor.glue_job_name
     assert retval["Job"]["Role"] == executor.role
     assert re.match(
-        "s3://example-bucket/redun/glue/oneshot-.*.py", retval["Job"]["Command"]["ScriptLocation"]
+        "s3://example-bucket/redun/glue/oneshot-.*.py",
+        retval["Job"]["Command"]["ScriptLocation"],
     )
 
     # Glue oneshot should now exist on S3.
@@ -334,15 +351,15 @@ def test_glue_submission_retry(describe_jobs_mock) -> None:
     executor.submit_pending_job.side_effect = get_retval
 
     # Submit a job and make sure it is in pending queue.
-    rjob = Job(task1(10))
+    rjob = Job(task1, task1(10))
     rjob.id = "123"
-    rjob.task = task1
     rjob.eval_hash = "hashbrowns"
+    rjob.args = ((10,), {})
 
-    executor.submit(rjob, [10], {})
+    executor.submit(rjob)
 
     wait_until(lambda: executor.submit_pending_job.call_count > 0)
-    assert list(executor.pending_glue_jobs) == [(rjob, [10], {})]
+    assert list(executor.pending_glue_jobs) == [rjob]
     assert len(executor.running_glue_jobs) == 0
 
     # Now when job submission works, it should be removed from pending queue
@@ -360,10 +377,20 @@ def test_glue_submission_retry(describe_jobs_mock) -> None:
 
     # Update mocks to indicate a job is done.
     executor.get_jobs.return_value = [
-        {"Id": "potato", "Arguments": {"--job-hash": "hashbrowns"}, "JobRunState": "SUCCEEDED"}
+        {
+            "Id": "potato",
+            "Arguments": {"--job-hash": "hashbrowns"},
+            "JobRunState": "SUCCEEDED",
+        }
     ]
     describe_jobs_mock.return_value = iter(
-        [{"Id": "potato", "Arguments": {"--job-hash": "hashbrowns"}, "JobRunState": "SUCCEEDED"}]
+        [
+            {
+                "Id": "potato",
+                "Arguments": {"--job-hash": "hashbrowns"},
+                "JobRunState": "SUCCEEDED",
+            }
+        ]
     )
 
     scheduler.batch_wait([rjob.id])
@@ -382,12 +409,12 @@ def test_glue_submit_job(submit_job_mock) -> None:
     scheduler = mock_scheduler()
     executor = mock_executor(scheduler, code_package=True)
 
-    job = Job(task1(10))
+    job = Job(task1, task1(10))
     job.id = "123"
-    job.task = task1
     job.eval_hash = "eval_hash"
+    job.args = ((10,), {})
 
-    executor.submit(job, [10], {})
+    executor.submit(job)
     assert executor.is_running
     assert File("s3://example-bucket/redun/jobs/eval_hash/input").exists()
 
