@@ -9,6 +9,7 @@ from typing import Any, List, cast
 from unittest.mock import Mock, patch
 
 import boto3
+import botocore
 import pytest
 from freezegun import freeze_time
 from moto import mock_logs
@@ -17,6 +18,7 @@ from sqlalchemy.orm import Session
 import redun
 from redun import File, Scheduler, task
 from redun.backends.db import CallNode, Execution, Job, RedunBackendDb, Value
+from redun.backends.db.query import find_file
 from redun.cli import (
     REDUN_CONFIG_ENV,
     CallGraphQuery,
@@ -24,8 +26,6 @@ from redun.cli import (
     RedunClientError,
     check_version,
     find_config_dir,
-    find_file,
-    get_abs_db_uri,
     get_config_dir,
     import_script,
     is_port_in_use,
@@ -35,7 +35,8 @@ from redun.cli import (
 from redun.executors.aws_batch import BATCH_LOG_GROUP
 from redun.executors.code_packaging import create_tar
 from redun.job_array import AWS_ARRAY_VAR
-from redun.scheduler import Traceback
+from redun.scheduler import SchedulerError, Traceback
+from redun.scheduler_config import get_abs_db_uri
 from redun.tags import ANY_VALUE
 from redun.tests.utils import assert_match_lines, use_tempdir
 from redun.utils import get_import_paths, pickle_dump
@@ -536,7 +537,10 @@ def call_hello2(func: Callable) -> str:
     assert client.execute(["redun", "run", "workflow.py", "main"]) == (10, True)
 
     # Parse argument to int.
-    assert client.execute(["redun", "run", "workflow.py", "main", "--x", "20"]) == (20, True)
+    assert client.execute(["redun", "run", "workflow.py", "main", "--x", "20"]) == (
+        20,
+        True,
+    )
 
     # Parse argument to int and bool.
     assert client.execute(
@@ -554,24 +558,60 @@ def call_hello2(func: Callable) -> str:
 
     # Parse argument as list of ints.
     assert client.execute(
-        ["redun", "run", "workflow.py", "multiply", "--numbers", "1", "2", "3", "--m", "30"]
+        [
+            "redun",
+            "run",
+            "workflow.py",
+            "multiply",
+            "--numbers",
+            "1",
+            "2",
+            "3",
+            "--m",
+            "30",
+        ]
     ) == [30, 60, 90]
 
     # Parse argument as list of bools.
     assert client.execute(
-        ["redun", "run", "workflow.py", "apply_not", "--values", "true", "False", "TRUE"]
+        [
+            "redun",
+            "run",
+            "workflow.py",
+            "apply_not",
+            "--values",
+            "true",
+            "False",
+            "TRUE",
+        ]
     ) == [False, True, False]
 
     # Parse argument as list of enums.
     assert client.execute(
-        ["redun", "run", "workflow.py", "color_to_string", "--colors", "Color.red", "blue"]
+        [
+            "redun",
+            "run",
+            "workflow.py",
+            "color_to_string",
+            "--colors",
+            "Color.red",
+            "blue",
+        ]
     ) == ["Color.red", "Color.blue"]
 
     # Confirm list args are validated against their type.
     with pytest.raises(ValueError, match="argument --colors: invalid Color value"):
         # black is not a color in the enum, should raise an error.
         assert client.execute(
-            ["redun", "run", "workflow.py", "color_to_string", "--colors", "Color.red", "black"]
+            [
+                "redun",
+                "run",
+                "workflow.py",
+                "color_to_string",
+                "--colors",
+                "Color.red",
+                "black",
+            ]
         )
 
     # Confirm optional args are not required by the CLI
@@ -592,17 +632,32 @@ def call_hello2(func: Callable) -> str:
 
     # Detect errors when parsing a function name.
     with pytest.raises(
-        ValueError, match="argument --func: invalid function value: 'bad_workflow.hello'"
+        ValueError,
+        match="argument --func: invalid function value: 'bad_workflow.hello'",
     ):
         client.execute(
-            ["redun", "run", "workflow.py", "call_hello2", "--func", "bad_workflow.hello"]
+            [
+                "redun",
+                "run",
+                "workflow.py",
+                "call_hello2",
+                "--func",
+                "bad_workflow.hello",
+            ]
         )
 
     with pytest.raises(
         ValueError, match="argument --func: invalid function value: 'workflow.bad_func'"
     ):
         client.execute(
-            ["redun", "run", "workflow.py", "call_hello2", "--func", "workflow.bad_func"]
+            [
+                "redun",
+                "run",
+                "workflow.py",
+                "call_hello2",
+                "--func",
+                "workflow.bad_func",
+            ]
         )
 
     with pytest.raises(ValueError, match="argument --func: invalid function value: 'hello'"):
@@ -675,8 +730,14 @@ def test_parse_func_path() -> None:
     """
     `parse_func_path()` should be able to parse the pattern 'file_or_module::function_name'.
     """
-    assert parse_func_path("workflow.py::setup_scheduler") == ("workflow.py", "setup_scheduler")
-    assert parse_func_path("workflow::setup_scheduler") == ("workflow", "setup_scheduler")
+    assert parse_func_path("workflow.py::setup_scheduler") == (
+        "workflow.py",
+        "setup_scheduler",
+    )
+    assert parse_func_path("workflow::setup_scheduler") == (
+        "workflow",
+        "setup_scheduler",
+    )
 
 
 @use_tempdir
@@ -754,7 +815,7 @@ setup_scheduler = workflow::setup_scheduler
 
 
 @use_tempdir
-def test_find_file(scheduler: Scheduler) -> None:
+def test_find_file(scheduler: Scheduler, session: Session) -> None:
     @task()
     def task1():
         file = File("out.txt")
@@ -763,8 +824,7 @@ def test_find_file(scheduler: Scheduler) -> None:
 
     scheduler.run(task1())
 
-    assert isinstance(scheduler.backend, RedunBackendDb)
-    row = find_file(scheduler.backend, "out.txt")
+    row = find_file(session, "out.txt")
     assert row
     file, job, kind = row
     assert file.path == "out.txt"
@@ -772,7 +832,7 @@ def test_find_file(scheduler: Scheduler) -> None:
 
 
 @use_tempdir
-def test_find_file_as_subvalue(scheduler: Scheduler) -> None:
+def test_find_file_as_subvalue(scheduler: Scheduler, session: Session) -> None:
     """
     We should be able to find Files that are subvalues.
     """
@@ -786,8 +846,7 @@ def test_find_file_as_subvalue(scheduler: Scheduler) -> None:
 
     scheduler.run(task1())
 
-    assert isinstance(scheduler.backend, RedunBackendDb)
-    result = find_file(scheduler.backend, "out.txt")
+    result = find_file(session, "out.txt")
     assert result
     file, job, kind = result
     assert file.path == "out.txt"
@@ -795,7 +854,7 @@ def test_find_file_as_subvalue(scheduler: Scheduler) -> None:
 
 
 @use_tempdir
-def test_find_file_arg(scheduler: Scheduler) -> None:
+def test_find_file_arg(scheduler: Scheduler, session: Session) -> None:
     """
     Find a File given as an Argument to a Task.
     """
@@ -812,8 +871,7 @@ def test_find_file_arg(scheduler: Scheduler) -> None:
 
     scheduler.run(main())
 
-    assert isinstance(scheduler.backend, RedunBackendDb)
-    result = find_file(scheduler.backend, "out.txt")
+    result = find_file(session, "out.txt")
     assert result
     file, job, kind = result
     assert file.path == "out.txt"
@@ -821,7 +879,7 @@ def test_find_file_arg(scheduler: Scheduler) -> None:
 
 
 @use_tempdir
-def test_find_file_arg_subvalue(scheduler: Scheduler) -> None:
+def test_find_file_arg_subvalue(scheduler: Scheduler, session: Session) -> None:
     """
     Find a File given as a Subvalue in an Argument to a Task.
     """
@@ -838,8 +896,7 @@ def test_find_file_arg_subvalue(scheduler: Scheduler) -> None:
 
     scheduler.run(main())
 
-    assert isinstance(scheduler.backend, RedunBackendDb)
-    result = find_file(scheduler.backend, "out.txt")
+    result = find_file(session, "out.txt")
     assert result
     file, job, kind = result
     assert file.path == "out.txt"
@@ -972,7 +1029,13 @@ def task1(x: int):
 
 
 @use_tempdir
-def test_run_tags() -> None:
+@patch("redun.cli.get_aws_user", side_effect=botocore.exceptions.NoCredentialsError())
+@patch(
+    "redun.cli.get_simple_aws_user",
+    side_effect=botocore.exceptions.NoCredentialsError(),
+)
+@patch("redun.cli.get_gcp_user", lambda: None)
+def test_run_tags(get_simple_aws_user_mock: Mock, get_aws_user_mock: Mock) -> None:
     """
     Run tags should be recorded on the execution.
     """
@@ -1126,7 +1189,6 @@ def task1(x: int):
 @patch("redun.cli.aws_describe_jobs")
 @use_tempdir
 def test_aws_logs(aws_describe_jobs_mock1, aws_describe_jobs_mock2) -> None:
-
     timestamp = int(time.time() * 1000)
     stream_name = "redun_aws_batch_example-redun-jd/default/6c939514f4054fdfb5ee65acc8aa4b07"
     job_descriptions = [
@@ -1233,6 +1295,40 @@ def task1(x: int):
     assert tags["project"] == ["my-project"]
 
 
+@use_tempdir
+def test_cloud_user_tags() -> None:
+    """
+    Run cloud (AWS, GCP) user tags should be recorded on the execution.
+    """
+    file = File("workflow.py")
+    file.write(
+        """
+from redun import task
+
+@task()
+def task1(x: int):
+    return x + 1
+"""
+    )
+
+    with patch("redun.cli.get_aws_user", lambda _: "arn:::::alice"), patch(
+        "redun.cli.get_simple_aws_user", lambda _: "alice"
+    ), patch("redun.cli.get_gcp_user", lambda: "alice@gmail.com"):
+        client = RedunClient()
+        client.execute(["redun", "run", "workflow.py", "task1", "--x", "10"])
+
+    scheduler = setup_scheduler()
+    assert scheduler.backend
+    backend = cast(RedunBackendDb, scheduler.backend)
+    assert backend.session
+
+    execution = backend.session.query(Execution).one()
+    tags = backend.get_tags([execution.id])[execution.id]
+    assert tags["user_aws"] == ["alice"]
+    assert tags["user_aws_arn"] == ["arn:::::alice"]
+    assert tags["user_gcp"] == ["alice@gmail.com"]
+
+
 def test_query(scheduler: Scheduler, backend: RedunBackendDb, session: Session) -> None:
     """
     CallGraphQuery should query records correctly.
@@ -1247,6 +1343,7 @@ def test_query(scheduler: Scheduler, backend: RedunBackendDb, session: Session)
     # All entities should be part of query.
     query = CallGraphQuery(session)
     assert len(list(query.all())) == 7
+    assert len(list(query.order_by("time").all())) == 7
     assert list(query.count()) == [
         ("Execution", 1),
         ("Job", 1),
@@ -1266,6 +1363,9 @@ def test_query(scheduler: Scheduler, backend: RedunBackendDb, session: Session)
     # Filter by type.
     execution = query.filter_types(["Execution"]).one()
     assert isinstance(execution, Execution)
+    # ensure the correct type is returned when ordering query is added
+    execution_ordered = query.order_by("time").filter_types(["Execution"]).one()
+    assert isinstance(execution_ordered, Execution)
 
     call_node = query.filter_types(["CallNode"]).one()
     assert isinstance(call_node, CallNode)
@@ -1396,6 +1496,7 @@ def test_value_subqueries(scheduler: Scheduler, backend: RedunBackendDb, session
 
     # Get the three executions.
     exec3, exec2, exec1 = list(query.filter_types(["Execution"]).order_by("time").all())
+    assert exec3.call_node.timestamp > exec2.call_node.timestamp > exec1.call_node.timestamp
 
     # We should be able to fetch the other record types from the execution.
     assert (
@@ -1668,7 +1769,13 @@ def main(x: int):
 
 
 @use_tempdir
-def test_cli_log_tags() -> None:
+@patch("redun.cli.get_aws_user", side_effect=botocore.exceptions.NoCredentialsError())
+@patch(
+    "redun.cli.get_simple_aws_user",
+    side_effect=botocore.exceptions.NoCredentialsError(),
+)
+@patch("redun.cli.get_gcp_user", lambda: None)
+def test_cli_log_tags(get_simple_aws_user_mock: Mock, get_aws_user_mock: Mock) -> None:
     """
     cli should allow querying the CallGraph for tags.
     """
@@ -1693,7 +1800,18 @@ def main(x: int) -> int:
 
     # Run example workflow.
     client = RedunClient()
-    client.execute(["redun", "run", "--user", "alice", "workflow.py", "task1", "--x", "10"])
+    client.execute(
+        [
+            "redun",
+            "run",
+            "--user",
+            "alice",
+            "workflow.py",
+            "task1",
+            "--x",
+            "10",
+        ]
+    )
 
     scheduler = setup_scheduler()
     assert scheduler.backend
@@ -2001,15 +2119,15 @@ def task1(x: int):
     return x + 1
 
 @task()
-def main(x: int):
-    return task1(x) + 1
+def main(x: int, y: int):
+    return task1(x) + 1 + y
 """
     )
 
     client = RedunClient()
 
-    result = client.execute(["redun", "run", "workflow.py", "main", "--x", "10"])
-    assert result == 12
+    result = client.execute(["redun", "run", "workflow.py", "main", "--x", "10", "--y", "5"])
+    assert result == 17
 
     assert client.scheduler
     scheduler = client.scheduler
@@ -2025,13 +2143,50 @@ def main(x: int):
         if job.parent_id:
             assert result == 11
         else:
-            assert result == 12
+            assert result == 17
 
     # Ensure that we can specify execution ids as well as job ids
     execution = backend.session.query(Execution).first()
     result = client.execute(["redun", "run", "--rerun", "workflow.py", execution.id])
+    assert result == 17
+
+    result = client.execute(["redun", "run", "--rerun", "workflow.py", execution.id, "--y", "10"])
+    assert result == 22
+
+
+@use_tempdir
+def test_run_packed_inputs() -> None:
+    """
+    Test that `run` accepts binary inputs w/ the --input flag.
+    """
+    file = File("workflow.py")
+    file.write(
+        """
+from redun import task
+
+redun_namespace = 'test'
+
+@task()
+def main(x: int, y: int = 0):
+    # Note: the default value y=0 is important to the test, this is a corner case of the CLI.
+    return x + y
+"""
+    )
+
+    client = RedunClient()
+
+    with File("inputs").open("wb") as out:
+        pickle_dump([(5,), {"y": 7}], out)
+
+    result = client.execute(["redun", "run", "workflow.py", "main", "--input", "inputs"])
     assert result == 12
 
+    # We should still allow overrides from the cmdline.
+    result = client.execute(
+        ["redun", "run", "workflow.py", "main", "--input", "inputs", "--y", "10"]
+    )
+    assert result == 15
+
 
 @pytest.mark.parametrize("executor", ["proc", "thread"])
 @use_tempdir
@@ -2061,23 +2216,337 @@ def main() -> int:
     this_pid = os.getpid()
     client = RedunClient()
     client.execute(
-        ["redun", "launch", "--executor", executor, "redun", "run", "workflow.py", "main"]
+        [
+            "redun",
+            "launch",
+            "--executor",
+            executor,
+            "redun",
+            "run",
+            "workflow.py",
+            "main",
+        ]
     )
 
-    if (sys.version_info.major, sys.version_info.minor) == (3, 8) and executor == "proc":
+    if (sys.version_info.major, sys.version_info.minor) == (
+        3,
+        8,
+    ) and executor == "proc":
         # Process executors are not shut down in py3.8, so it may still be running.
         # This is expected but we skip the test rather than try to wait it out
         return
-    assert client.scheduler
-    scheduler = client.scheduler
-    assert scheduler.backend
-    backend = cast(RedunBackendDb, scheduler.backend)
-    assert backend.session
-    value = backend.session.query(Value).filter(Value.type != "redun.Task").one()
+
+    parser = client.get_command_parser()
+    args, extra_args = parser.parse_known_args([])
+    value = client.get_session(args=args).query(Value).filter(Value.type != "redun.Task").one()
     assert value.value_parsed
     assert value.value_parsed != this_pid
 
 
+@pytest.fixture(scope="module")
+def context_workflow():
+    body = """
+import os
+import redun
+
+@redun.task
+def main(context: dict = redun.get_context("my_tool", None)) -> dict:
+    return context
+"""
+    return File("workflow.py").write(body)
+
+
+@use_tempdir
+def test_both_context_and_context_file_present(context_workflow) -> None:
+    """
+    Ensure that an error is raised if both context and context file are present.
+    """
+    config_string = """
+[scheduler]
+context_file = context.json
+context = {}
+"""
+    File(".redun/redun.ini").write(config_string)
+    client = RedunClient()
+    with pytest.raises(SchedulerError, match="Both context and context_file specified in config!"):
+        client.execute(["redun", "run", "workflow.py", "main"])
+
+
+@use_tempdir
+def test_context_file_in_redun_config(context_workflow) -> None:
+    """
+    Ensure that context_file option can be used in redun config.
+    """
+    config_string = """
+[scheduler]
+context_file = context.json
+"""
+    File(".redun/redun.ini").write(config_string)
+    File(".redun/context.json").write(
+        """{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    )
+    client = RedunClient()
+    result = client.execute(["redun", "run", "workflow.py", "main"])
+    assert result == {"a": 2, "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_override(context_workflow) -> None:
+    """
+    Ensure that the context from redun.ini can be overridden by the context file.
+    """
+    config_string = """
+[scheduler]
+context =
+  {
+    "my_tool": {
+      "a": 1,
+      "b": "foo"
+    }
+  }
+"""
+    context_override = """
+{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    File(".redun/redun.ini").write(config_string)
+    File("context.json").write(context_override)
+    client = RedunClient()
+    result = client.execute(
+        ["redun", "run", "--context-file", "context.json", "workflow.py", "main"]
+    )
+    assert result == {"a": 2, "b": "foo", "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_config_relative(context_workflow) -> None:
+    """
+    Ensure that when the context file is a relative path specified in redun.ini, it is searched in
+    the directory relative to the .redun config dir.
+    """
+    config_string = """
+[scheduler]
+context_file = ../context.json
+"""
+    File(".redun/redun.ini").write(config_string)
+    File("context.json").write(
+        """{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    )
+    client = RedunClient()
+    result = client.execute(["redun", "run", "workflow.py", "main"])
+    assert result == {"a": 2, "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_config_absolute(context_workflow) -> None:
+    """
+    Ensure that when the context file is an absolute path specified in redun.ini, it is used as is.
+    """
+    context_file_path = f"{os.getcwd()}/context.json"
+    assert os.path.isabs(context_file_path)
+
+    config_string = f"""
+[scheduler]
+context_file = {context_file_path}
+"""
+    File(".redun/redun.ini").write(config_string)
+    File(context_file_path).write(
+        """{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    )
+    client = RedunClient()
+    result = client.execute(["redun", "run", "workflow.py", "main"])
+    assert result == {"a": 2, "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_arg_relative(context_workflow) -> None:
+    """
+    Ensure that when the context file is a relative path specified as --context-file argument,
+    it is searched in the current working directory.
+    """
+    File("context.json").write(
+        """{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    )
+    client = RedunClient()
+    result = client.execute(
+        ["redun", "run", "--context-file", "context.json", "workflow.py", "main"]
+    )
+    assert result == {"a": 2, "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_arg_absolute(context_workflow) -> None:
+    """
+    Ensure that when the context file is an absolute path specified as --context-file argument,
+    it is used as is.
+    """
+    context_file_path = f"{os.getcwd()}/context.json"
+    assert os.path.isabs(context_file_path)
+
+    File(context_file_path).write(
+        """{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    )
+    client = RedunClient()
+    result = client.execute(
+        ["redun", "run", "--context-file", context_file_path, "workflow.py", "main"]
+    )
+    assert result == {"a": 2, "c": "bar"}
+
+
+@use_tempdir
+def test_context_option_override(context_workflow) -> None:
+    """
+    Ensure that the context from redun.ini can be overridden by the context option.
+    """
+    config_string = """
+[scheduler]
+context =
+  {
+    "my_tool": {
+      "a": 1,
+      "b": "foo"
+    }
+  }
+"""
+    File(".redun/redun.ini").write(config_string)
+    client = RedunClient()
+    result = client.execute(
+        [
+            "redun",
+            "run",
+            "--context",
+            "my_tool.a=2",
+            "--context",
+            "my_tool.c=bar",
+            "workflow.py",
+            "main",
+        ]
+    )
+    assert result == {"a": 2, "b": "foo", "c": "bar"}
+
+
+@use_tempdir
+def test_context_file_and_option_override(context_workflow) -> None:
+    """
+    Ensure that the context from redun.ini can be overridden by the context file and option.
+    Overrides from --context should take precedence over the context file and the context file
+    should take precedence over the redun.ini context.
+    """
+    config_string = """
+[scheduler]
+context =
+  {
+    "my_tool": {
+      "a": 1,
+      "b": "foo"
+    }
+  }
+"""
+    context_override = """
+{
+    "my_tool": {
+        "a": 2,
+        "c": "bar"
+    }
+}
+"""
+    File(".redun/redun.ini").write(config_string)
+    File("context.json").write(context_override)
+    client = RedunClient()
+    result = client.execute(
+        [
+            "redun",
+            "run",
+            "--context-file",
+            "context.json",
+            "--context",
+            "my_tool.a=3",
+            "--context",
+            "my_tool.d=baz",
+            "workflow.py",
+            "main",
+        ]
+    )
+    assert result == {"a": 3, "b": "foo", "c": "bar", "d": "baz"}
+
+
+@use_tempdir
+def test_context_update_context_precedence() -> None:
+    """
+    Ensure that update_context takes precedence over any other option.
+    """
+    config_string = """
+[scheduler]
+context =
+  {
+    "my_tool": {
+      "a": 1,
+      "b": "foo"
+    }
+  }
+"""
+    workflow = """
+import os
+import redun
+
+@redun.task
+def inner(context: dict = redun.get_context("my_tool", None)) -> dict:
+    return context
+
+@redun.task
+def main() -> dict:
+    return inner.update_context(my_tool={"a": 3, "c": "bar"})()
+"""
+    File(".redun/redun.ini").write(config_string)
+    File("workflow.py").write(workflow)
+    client = RedunClient()
+    result = client.execute(
+        [
+            "redun",
+            "run",
+            "--context",
+            "my_tool.a=2",
+            "workflow.py",
+            "main",
+        ]
+    )
+    assert result == {"a": 3, "b": "foo", "c": "bar"}
+
+
 @pytest.mark.parametrize("executor", ["proc", "thread"])
 @use_tempdir
 def test_wait_launch(executor: str) -> None:
