@@ -1,14 +1,22 @@
 from collections import Counter
+from datetime import timedelta, timezone
 from typing import Callable, Dict, List, Set, cast
 from unittest import mock
 from unittest.mock import patch
 
 import pytest
+from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import Session
 from sqlalchemy.sql.expression import cast as sa_cast
 
 from redun import Handle, Scheduler, catch, cond, task
-from redun.backends.db import JSON, REDUN_DB_VERSIONS, CallNode, DBVersionInfo, Execution
+from redun.backends.db import (
+    JSON,
+    REDUN_DB_VERSIONS,
+    CallNode,
+    DBVersionInfo,
+    Execution,
+)
 from redun.backends.db import Handle as DbHandle
 from redun.backends.db import (
     HandleEdge,
@@ -20,7 +28,7 @@ from redun.backends.db import (
     RedunVersion,
     RedunVersionError,
     Tag,
-    TagEntityType,
+    TagEntity,
     Value,
     parse_db_version,
 )
@@ -28,7 +36,7 @@ from redun.config import Config, create_config_section
 from redun.file import Dir, File
 from redun.functools import seq
 from redun.tests.utils import use_tempdir
-from redun.utils import PreviewClass
+from redun.utils import PreviewClass, utcnow
 
 
 def test_cache_db() -> None:
@@ -57,16 +65,43 @@ def test_max_value_size() -> None:
 
     backend = RedunBackendDb(db_uri="sqlite:///:memory:", config=config)
     backend.load()
-    small_object = "J" * 10000
+    large_object = "J" * 10000
 
     with pytest.raises(RedunDatabaseError):
-        backend.record_value(small_object)
+        backend.record_value(large_object)
+
+
+def test_race_condition_record_value() -> None:
+    """
+    Test proper handling of a dirty read when a recorded value exists even though it didn't
+    just a couple of lines of code before.
+    """
+    backend = RedunBackendDb(db_uri="sqlite:///:memory:")
+    backend.load()
+    proper_commit = backend.session.commit  # type: ignore
+
+    def commit_but_raise_anyway(*args, **kwargs):
+        proper_commit(*args, **kwargs)
+        raise IntegrityError("UniquenessViolated", "UniquenessViolated", "UniquenessViolated")
+
+    backend.session.commit = mock.MagicMock(side_effect=commit_but_raise_anyway)  # type: ignore
+    # record value should gracefully handle the IntegrityError and return the value_hash
+    value_hash1 = backend.record_value("myvalue")
+
+    assert backend.get_value(value_hash1) == ("myvalue", True)
+
+    backend.session.commit = mock.MagicMock(  # type: ignore
+        side_effect=IntegrityError("OtherIssue", "OtherIssue", "OtherIssue")
+    )
+
+    with pytest.raises(IntegrityError):
+        backend.record_value("my-other-value")
 
 
 def test_call_graph_db() -> None:
     backend = RedunBackendDb(db_uri="sqlite:///:memory:")
     backend.load()
-    backend.current_task_hashes = {
+    task_hashes = {
         "task_hash1",
         "task_hash2",
         "task_hash3",
@@ -102,7 +137,7 @@ def test_call_graph_db() -> None:
     )
 
     # Get a node.
-    call_node = backend._get_call_node("task_hash3", "args_hash3")
+    call_node = backend._get_call_node("task_hash3", "args_hash3", task_hashes)
     assert call_node
     assert call_node.task_name == "mytask3"
 
@@ -111,7 +146,7 @@ def test_call_graph_db() -> None:
     assert call_node.children[1].task_name == "mytask2"
 
     # Get another node.
-    call_node = backend._get_call_node("task_hash1", "args_hash1")
+    call_node = backend._get_call_node("task_hash1", "args_hash1", task_hashes)
     assert call_node
     assert call_node.task_name == "mytask1"
 
@@ -153,6 +188,15 @@ def test_handle_db() -> None:
     assert not backend.is_valid_handle(handle4)
 
 
+def test_handle_null():
+    backend = RedunBackendDb(db_uri="sqlite:///:memory:")
+    backend.load()
+
+    handle = Handle("conn")
+    # Unsaved handle should not be considered valid
+    assert not backend.is_valid_handle(handle)
+
+
 def test_scheduler_db():
     """
     Scheduler should be able to use the database Backend.
@@ -269,7 +313,8 @@ def test_task_args(scheduler: Scheduler, session: Session) -> None:
 
     last_exec = session.query(Execution).one()
     [arg1, arg2, arg3, arg4] = sorted(
-        last_exec.call_node.arguments, key=lambda arg: arg.arg_key or str(arg.arg_position)
+        last_exec.call_node.arguments,
+        key=lambda arg: arg.arg_key or str(arg.arg_position),
     )
 
     # Positional arguments.
@@ -319,9 +364,10 @@ def test_multiple_expr(scheduler: Scheduler) -> None:
     assert task_calls == ["task1"]
 
     # Now let's test the cache.
-    with mock.patch.object(scheduler, "get_cache", wraps=scheduler.get_cache) as get_cache:
+    with mock.patch.object(scheduler, "_get_cache", wraps=scheduler._get_cache) as get_cache:
         assert scheduler.run(main(1)) == [[1, 1], [2, 1]]
-        assert get_cache.call_count == 4
+        # Depending on timing, we'll get a CSE from pending or the backend.
+        assert 4 <= get_cache.call_count <= 5
 
     # Get root call node.
     backend = scheduler.backend
@@ -385,6 +431,28 @@ def test_parent_job_simple(scheduler: Scheduler, session: Session) -> None:
     assert last_exec.job.child_jobs[0].task.name == "task1"
 
 
+def test_job_time_utc(scheduler: Scheduler, session: Session) -> None:
+    """
+    Parent job start_time and end_time should be timezone aware and UTC.
+    """
+
+    @task()
+    def task1():
+        return 10
+
+    now = utcnow()
+    assert scheduler.run(task1()) == 10
+
+    job = session.query(Job).one()
+    assert job.start_time.tzinfo == timezone.utc
+    assert job.end_time.tzinfo == timezone.utc
+    assert job.call_node.timestamp.tzinfo == timezone.utc
+
+    # Check that we have not somehow misconverted timezones and caused a difference
+    # on the order of several hours.
+    assert abs(now - job.start_time) < timedelta(seconds=5)
+
+
 def test_parent_job_simple_arg(scheduler: Scheduler, session: Session) -> None:
     """
     Parent job should be set for subtasks even through SimpleExpression as arguments.
@@ -561,6 +629,11 @@ def test_dir_subvalues(scheduler: Scheduler, session: Session) -> None:
     paths = {subvalue.value_parsed.path for subvalue in subvalues}
     assert paths == {"dir/file1", "dir/file2"}
 
+    # they are also accessible by "subfiles" property
+    subfiles = arg_value.subfiles
+    subfile_paths = {subfile.value.value_parsed.path for subfile in subfiles}
+    assert subfile_paths == {"dir/file1", "dir/file2"}
+
 
 @use_tempdir
 def test_nested_dir_subvalues(scheduler: Scheduler, session: Session) -> None:
@@ -619,6 +692,29 @@ def test_execution_job(scheduler: Scheduler, session: Session) -> None:
     assert {job.task.name for job in last_exec.jobs} == {"task1", "main"}
 
 
+def test_updated_time(scheduler: Scheduler, session: Session) -> None:
+    """
+    We should be able to query an Execution's updated_time,
+    which should be somewhere between the start and end times.
+    """
+
+    @task()
+    def task1(x):
+        return x * 2
+
+    @task()
+    def main(x):
+        return task1(x + 1)
+
+    start_time = utcnow()
+    assert scheduler.run(main(20)) == 42
+    end_time = utcnow()
+
+    last_exec = session.query(Execution).one()
+    assert last_exec.updated_time is not None
+    assert start_time < last_exec.updated_time < end_time
+
+
 def test_job_status(scheduler: Scheduler, session: Session) -> None:
     """
     We should be able to classify a Job's status.
@@ -647,6 +743,23 @@ def test_job_status(scheduler: Scheduler, session: Session) -> None:
     assert Counter(job.status for job in exec2.jobs) == Counter({"FAILED": 2, "CACHED": 1})
 
 
+def test_failed_job_end_time(scheduler: Scheduler, session: Session) -> None:
+    """
+    A failed job should have an end_time.
+    """
+
+    @task()
+    def main(x):
+        assert x != 0
+        return x + 1
+
+    with pytest.raises(AssertionError):
+        scheduler.run(main(0))
+
+    [exec1] = session.query(Execution).all()
+    assert exec1.job.status == "FAILED" and exec1.job.end_time is not None
+
+
 def test_catch_downstream(scheduler: Scheduler) -> None:
     """
     Result from catch should properly propagate to downstream tasks.
@@ -773,7 +886,11 @@ def test_seq_dataflow(scheduler: Scheduler, session: Session) -> None:
     assert scheduler.run(main()) == [2, 3, 4]
 
     exec1 = session.query(Execution).one()
-    assert [job.call_node.value.value_parsed for job in exec1.job.child_jobs] == [2, 3, 4]
+    assert [job.call_node.value.value_parsed for job in exec1.job.child_jobs] == [
+        2,
+        3,
+        4,
+    ]
 
 
 def test_cond_dataflow(scheduler: Scheduler, session: Session) -> None:
@@ -985,7 +1102,6 @@ def test_db_backend_clone() -> None:
     clone = backend.clone()
     assert clone.engine is backend.engine
     assert clone.current_execution is backend.current_execution
-    assert clone.current_task_hashes is backend.current_task_hashes
 
     assert clone.session is not backend.session
 
@@ -999,7 +1115,7 @@ def test_json_field(scheduler: Scheduler, session: Session) -> None:
     session.add(
         Tag(
             tag_hash="111",
-            entity_type=TagEntityType.Value,
+            entity_type=TagEntity.Value,
             entity_id="1",
             key="params",
             value="hello",
@@ -1012,7 +1128,13 @@ def test_json_field(scheduler: Scheduler, session: Session) -> None:
 
     # Add and filter an int value.
     session.add(
-        Tag(tag_hash="222", entity_type=TagEntityType.Value, entity_id="2", key="params", value=10)
+        Tag(
+            tag_hash="222",
+            entity_type=TagEntity.Value,
+            entity_id="2",
+            key="params",
+            value=10,
+        )
     )
     session.commit()
 
@@ -1023,7 +1145,7 @@ def test_json_field(scheduler: Scheduler, session: Session) -> None:
     session.add(
         Tag(
             tag_hash="333",
-            entity_type=TagEntityType.Value,
+            entity_type=TagEntity.Value,
             entity_id="2",
             key="params",
             value=[1, 2, 3],
@@ -1042,7 +1164,7 @@ def test_json_field(scheduler: Scheduler, session: Session) -> None:
     session.add(
         Tag(
             tag_hash="444",
-            entity_type=TagEntityType.Value,
+            entity_type=TagEntity.Value,
             entity_id="2",
             key="params",
             value={"a": 1, "b": 2},
