@@ -1,7 +1,7 @@
 import os
 import shlex
 import shutil
-from unittest.mock import patch
+from unittest.mock import MagicMock, patch
 
 import boto3
 import pytest
@@ -9,7 +9,20 @@ import vcr
 
 import redun.file
 from redun import File, Scheduler, ShardedS3Dataset, task
-from redun.file import Dir, StagingDir, StagingFile, get_filesystem, glob_file
+from redun.file import (
+    AzureBlobFileSystem,
+    ContentDir,
+    ContentFile,
+    ContentStagingFile,
+    Dir,
+    IDir,
+    IFile,
+    IStagingFile,
+    StagingDir,
+    StagingFile,
+    get_filesystem,
+    glob_file,
+)
 from redun.tests.utils import mock_s3, use_tempdir
 from redun.tools import copy_file
 from redun.value import get_type_registry
@@ -51,7 +64,7 @@ def test_encoding() -> None:
     """
     File read/write/open should support encodings.
     """
-    text = "hello \u1F600"
+    text = "hello \u1f600"
 
     # Read and write utf-8.
     file = File("hello.txt")
@@ -79,7 +92,6 @@ def test_encoding() -> None:
 
 @use_tempdir
 def test_dir_api() -> None:
-
     dir = Dir("hi")
     assert dir.rel_path(os.getcwd()) == ".."
 
@@ -141,7 +153,6 @@ def test_dir_subvalues() -> None:
 
 @use_tempdir
 def test_workflow(scheduler: Scheduler) -> None:
-
     task_calls = []
 
     @task()
@@ -187,7 +198,6 @@ def test_workflow(scheduler: Scheduler) -> None:
 
 @use_tempdir
 def test_infile(scheduler: Scheduler) -> None:
-
     task_calls = []
 
     @task()
@@ -934,7 +944,8 @@ def test_non_existent_file_error() -> None:
     """
 
     with pytest.raises(
-        FileNotFoundError, match=r"\[Errno 2\] No such file or directory\. not_exist_file"
+        FileNotFoundError,
+        match=r"\[Errno 2\] No such file or directory\. not_exist_file",
     ):
         File("not_exist_file").open()
 
@@ -965,3 +976,157 @@ def test_readonly_file_error() -> None:
     os.system("chmod 400 readonly_file")
     with pytest.raises(PermissionError, match=r"\[Errno 13\] Permission denied\. readonly_file"):
         File("readonly_file").open("w")
+
+
+@use_tempdir
+def test_ifile() -> None:
+    """
+    IFile hash should only depend on path.
+    """
+
+    # First confirm that regular files update hash based on contents and timestamp.
+    file1 = File("file")
+    file1.write("1")
+
+    file2 = File("file")
+    file2.write("22")
+
+    assert file1.get_hash() != file2.get_hash()
+
+    # IFile hashes only depend on path.
+    file1 = IFile("ifile")
+    file1.write("1")
+
+    file2 = IFile("ifile")
+    file2.write("22")
+
+    assert file1.get_hash() == file2.get_hash()
+
+
+@use_tempdir
+def test_ifile_classes() -> None:
+    """
+    IFile methods should return related objects of the right type.
+    """
+    assert isinstance(IFile("file").stage("local"), IStagingFile)
+    assert isinstance(IDir("dir").file("file"), IFile)
+
+    IFile("dir/a").touch()
+    IFile("dir/b").touch()
+    assert all(isinstance(file, IFile) for file in IDir("dir"))
+
+    assert IFile("dir/a").stage("a").render_stage() == "cp dir/a a"
+    assert IFile("dir/a").stage("a").render_unstage() == "cp a dir/a"
+    assert IDir("dir").stage("dir2").render_stage() == "cp -r dir dir2"
+    assert IDir("dir").stage("dir2").render_unstage() == "cp -r dir2 dir"
+
+
+@use_tempdir
+def test_content_file() -> None:
+    """
+    ContentFile hash should only depend on path and content (not timestamp).
+    """
+
+    # Hash should only depend on path.
+    file1 = ContentFile("file")
+    file1.write("hi")
+
+    file2 = ContentFile("file")
+    file2.write("hi")
+
+    assert file1.get_hash() == file2.get_hash()
+
+    # Hash should update with content.
+    file2.write("bye")
+    assert file1.get_hash() != file2.get_hash()
+
+    # Hash should go back after update.
+    file2.write("hi")
+    assert file1.get_hash() == file2.get_hash()
+
+    # Different content should change the hash.
+    file1 = ContentFile("file")
+    file1.write("hi")
+
+    file2 = ContentFile("file")
+    file2.write("bye")
+
+    assert file1.get_hash() != file2.get_hash()
+
+
+@use_tempdir
+def test_content_file_classes() -> None:
+    """
+    ContentFile methods should return related objects of the right type.
+    """
+    assert isinstance(ContentFile("file").stage("local"), ContentStagingFile)
+    assert isinstance(ContentDir("dir").file("file"), ContentFile)
+
+    ContentFile("dir/a").touch()
+    ContentFile("dir/b").touch()
+    assert all(isinstance(file, ContentFile) for file in ContentDir("dir"))
+
+
+@patch("adlfs.AzureBlobFileSystem")
+@patch("redun.azure_utils.get_az_credential")
+def test_azure_glob(_, adlfs_mock):
+    glob_mock = MagicMock()
+    glob_mock.return_value = [
+        "container1/sample/path/",
+        "container1/sample/path/file1.txt",
+        "container1/sample/path/file2.csv",
+    ]
+
+    adlfs_mock.return_value.glob = glob_mock
+    adlfs_mock.return_value.account_name = "myaccount"
+
+    fs = AzureBlobFileSystem()
+
+    result = fs.glob("az://container1@myaccount.blob.core.windows.net/sample/path/**")
+    assert result == [
+        "az://container1@myaccount.blob.core.windows.net/sample/path/",
+        "az://container1@myaccount.blob.core.windows.net/sample/path/file1.txt",
+        "az://container1@myaccount.blob.core.windows.net/sample/path/file2.csv",
+    ]
+
+
+@patch("adlfs.AzureBlobFileSystem")
+@patch("redun.azure_utils.get_az_credential")
+def test_azure_list_files_in_dir(_, adlfs_mock):
+    glob_mock = MagicMock()
+    glob_mock.return_value = [
+        "container1/sample/path/",
+        "container1/sample/path/file1.txt",
+        "container1/sample/path/file2.csv",
+    ]
+
+    def mock_isfile_func(path):
+        return path[-1] != "/"
+
+    adlfs_mock.return_value.glob = glob_mock
+    adlfs_mock.return_value.account_name = "myaccount"
+    adlfs_mock.return_value.isfile = MagicMock(side_effect=mock_isfile_func)
+
+    dir = redun.Dir("az://container1@myaccount.blob.core.windows.net/sample/path/")
+
+    result = {file.path for file in dir.files()}
+    assert result == {
+        "az://container1@myaccount.blob.core.windows.net/sample/path/file1.txt",
+        "az://container1@myaccount.blob.core.windows.net/sample/path/file2.csv",
+    }
+
+
+def test_azure_get_account_name_from_path():
+    fs = AzureBlobFileSystem()
+    result = fs.get_account_name_from_path(
+        "az://mycontainer@myawesomestorage.blob.core.windows.net/my/path"
+    )
+
+    assert result == "myawesomestorage"
+
+
+@patch("redun.azure_utils.get_az_credential")
+def test_azure_custom_credential_passed_to_fs(credential_mock):
+    fs = AzureBlobFileSystem()
+    internal_fs = fs.get_fs_for_path("az://mycontainer@myawesomestorage.blob.core.windows.net")
+    assert internal_fs.sync_credential == credential_mock.return_value
