@@ -8,65 +8,6 @@ from redun.functools import compose
 from redun.task import PartialTask
 from redun.value import get_type_registry
 
-NULL = object()
-
-
-@task(name="map", namespace="redun")
-def task_map(a_task, values):
-    return [a_task(value) for value in values]
-
-
-# Monkey patch Expression with map.
-def expression_map(self, a_task):
-    return TaskExpression("redun.map", (a_task, self), {})
-
-
-Expression.map = expression_map  # type: ignore
-
-
-@task(name="reduce", namespace="redun")
-def task_reduce(a_task, values, init=NULL):
-    values = iter(values)
-    if init is NULL:
-        result = next(values)
-    else:
-        result = init
-    for value in values:
-        result = a_task(result, value)
-    return result
-
-
-def expression_reduce(self, a_task, init=NULL):
-    return TaskExpression("redun.reduce", (a_task, self, init), {})
-
-
-Expression.reduce = expression_reduce  # type: ignore
-
-
-def test_expression_map(scheduler):
-    @task()
-    def get_list():
-        return [1, 2, 3]
-
-    @task()
-    def inc(x):
-        return x + 1
-
-    assert scheduler.run(get_list().map(inc)) == [2, 3, 4]
-
-
-def test_expression_reduce(scheduler):
-    @task()
-    def get_list():
-        return [1, 2, 3]
-
-    @task()
-    def add(a, b):
-        return a + b
-
-    assert scheduler.run(get_list().reduce(add)) == 6
-    assert scheduler.run(get_list().reduce(add, 10)) == 16
-
 
 def test_partial(scheduler: Scheduler) -> None:
     """
@@ -103,6 +44,21 @@ def test_partial(scheduler: Scheduler) -> None:
     assert scheduler.run(add.partial(a=1)(b=10)) == 11
 
 
+def test_partial_override(scheduler: Scheduler) -> None:
+    """
+    We should be able override previously specified arguments when calling a PartialTask.
+    """
+
+    @task
+    def add(a: int, b: int) -> int:
+        return a + b
+
+    add2 = add.partial(a=2)
+
+    # Override a=2 with a=3.
+    assert scheduler.run(add2(a=3, b=3)) == 6
+
+
 def test_partial_serialize() -> None:
     """
     PartialTask should serialize and deserialize.
@@ -172,17 +128,17 @@ def test_partial_options(scheduler: Scheduler) -> None:
 
     assert not add.has_task_option("memory")
     assert add.get_task_option("memory") is None
-    assert 1 == add.get_task_option("memory", 1)
+    assert add.get_task_option("memory", 1) == 1
 
     inc = add.partial(1).options(memory=4)
-    inc = cast(PartialTask, inc)
 
     # Should be recorded on task.
-    assert inc.task.has_task_option("memory")
-    assert 4 == inc.task.get_task_option("memory")
-    assert 4 == inc.task.get_task_option("memory", 1)
+    assert inc.has_task_option("memory")
+    assert inc.get_task_option("memory") == 4
+    assert inc.get_task_option("memory", 1) == 4
 
     # When we call the PartialTask, the task_options should propagate to the Expression.
+    inc = cast(PartialTask, inc)
     expr: Expression[int] = inc(2)
     assert expr.task_expr_options == {"memory": 4}
 
@@ -234,8 +190,7 @@ def test_partial_partial_options(scheduler: Scheduler) -> None:
 
     # Options should be present on PartialTask.
     part = add3.partial(1).partial(2).options(memory=4)
-    part = cast(PartialTask, part)
-    assert part.task.get_task_options() == {"memory": 4}
+    assert part.get_task_options() == {"memory": 4}
 
     # Options should propagate to Expression.
     expr: Expression[int] = part(3)
