@@ -159,9 +159,7 @@ def test_python_script_task(scheduler: Scheduler) -> None:
         return """
         #!/usr/bin/env python
         print('Hello, {message}!')
-        """.format(
-            message=message
-        )
+        """.format(message=message)
 
     assert scheduler.run(task1("World")) == b"Hello, World!\n"
 
@@ -194,6 +192,15 @@ def test_default_shell(scheduler: Scheduler) -> None:
     assert scheduler.run(result) == b"ok\n"
 
 
+def test_script_list(scheduler: Scheduler) -> None:
+    """
+    script() should accept lists as well.
+    """
+    # Note: The double space should be preserved since each element is shell quoted.
+    result = script(["echo", "hello  world"])
+    assert scheduler.run(result) == b"hello  world\n"
+
+
 def test_script_error(scheduler: Scheduler) -> None:
     """
     Scripts should propagate their errors.
