@@ -8,10 +8,19 @@ import pytest
 import redun
 from redun import Task, task
 from redun.expression import Expression
-from redun.scheduler import set_arg_defaults
-from redun.task import Func, get_task_registry, get_tuple_type_length, wraps_task
+from redun.namespace import compute_namespace
+from redun.scheduler import Scheduler, set_arg_defaults
+from redun.task import (
+    CacheCheckValid,
+    CacheScope,
+    Func,
+    get_task_registry,
+    get_tuple_type_length,
+    wraps_task,
+)
 from redun.tests.task_test_helpers import square_task
 from redun.utils import get_func_source, pickle_dumps
+from redun.value import get_type_registry
 
 
 def test_task_config() -> None:
@@ -23,14 +32,14 @@ def test_task_config() -> None:
     def my_task():
         return 10
 
-    assert Task._compute_namespace(my_task) == my_task.namespace
-    assert my_task.fullname == Task._format_fullname(Task._compute_namespace(my_task), "my_task")
+    assert compute_namespace(my_task) == my_task.namespace
+    assert my_task.fullname == Task._format_fullname(compute_namespace(my_task), "my_task")
 
     @task(name="custom_name", namespace="my_namespace", load_module="custom.module")
     def my_task2():
         return 10
 
-    assert Task._compute_namespace(my_task2, namespace="my_namespace") == my_task2.namespace
+    assert compute_namespace(my_task2, namespace="my_namespace") == my_task2.namespace
     assert my_task2.load_module == "custom.module"
 
     redun.namespace("my_namespace2")
@@ -39,7 +48,7 @@ def test_task_config() -> None:
     def my_task3():
         return 10
 
-    assert Task._compute_namespace(my_task3) == my_task3.namespace
+    assert compute_namespace(my_task3) == my_task3.namespace
 
     redun.namespace()
 
@@ -47,7 +56,7 @@ def test_task_config() -> None:
     assert my_task.fullname == "my_task"
     assert my_task.load_module == "redun.tests.test_tasks"
     assert my_task.hash
-    assert my_task.func
+    assert callable(my_task.func)
     assert (
         my_task.source
         == """\
@@ -450,8 +459,8 @@ def test_tuple_type_length() -> None:
     assert get_tuple_type_length(Tuple[int, int]) == 2
     assert get_tuple_type_length(Tuple[int, int, int]) == 3
     assert get_tuple_type_length(Tuple[()]) == 0
-    assert get_tuple_type_length(Tuple[Any, ...]) is None
     assert get_tuple_type_length(Tuple) is None
+    assert get_tuple_type_length(Tuple[Any, ...]) is None
 
     User = namedtuple("User", ["name", "age", "friend"])
     assert get_tuple_type_length(User) == 3
@@ -476,7 +485,6 @@ def test_wraps_task() -> None:
         @wraps_task(wrapper_hash_includes=[offset])
         def _doubled_task(inner_task: Task) -> Callable[[Task[Func]], Func]:
             def do_doubling(*task_args, **task_kwargs) -> Any:
-
                 return (
                     inner_task.func(*task_args, **task_kwargs)
                     + inner_task.func(*task_args, **task_kwargs)
@@ -523,7 +531,6 @@ def test_wraps_task() -> None:
         @wraps_task(wrapper_name="renamed_task")
         def _listify_task(inner_task: Task) -> Callable[[Task[Func]], Func]:
             def make_list(*task_args, copies=1, **task_kwargs) -> Any:
-
                 return [inner_task] * copies
 
             return make_list
@@ -539,7 +546,11 @@ def test_wraps_task() -> None:
 
     float_task_inner = get_task_registry()._tasks["renamed_task.float_task"]
     assert float_task.func() == [float_task_inner]
-    assert float_task.func(copies=3) == [float_task_inner, float_task_inner, float_task_inner]
+    assert float_task.func(copies=3) == [
+        float_task_inner,
+        float_task_inner,
+        float_task_inner,
+    ]
 
     # 4. Try using a wrapper that is not defined in this module, so we can tell whether the
     # `load_module` points here, as it should. This is more likely representative of real use,
@@ -564,7 +575,10 @@ def test_wraps_task() -> None:
         return -2
 
     extra_nested_middle_task = get_task_registry()._tasks["renamed_task.extra_nested"]
-    assert extra_nested.func(copies=2) == [extra_nested_middle_task, extra_nested_middle_task]
+    assert extra_nested.func(copies=2) == [
+        extra_nested_middle_task,
+        extra_nested_middle_task,
+    ]
     assert extra_nested_middle_task.func() == -3
     assert get_task_registry()._tasks["_doubled_task.renamed_task.extra_nested"].func() == -2
 
@@ -618,3 +632,133 @@ def test_wraps_task() -> None:
     # After all our renaming, make sure our registry is still a correct index of task names.
     for task_name, task_ in get_task_registry()._tasks.items():
         assert task_name == task_.fullname
+
+
+def test_wraps_task_inner_task(scheduler: Scheduler) -> None:
+    """
+    wraps_task should maintain a reference to the inner task.
+    """
+
+    def doubled_task(offset: int = 0) -> Callable[[Func], Task[Func]]:
+        @wraps_task(wrapper_hash_includes=[offset])
+        def _doubled_task(inner_task: Task) -> Callable[[Task[Func]], Func]:
+            def do_doubling(*task_args, **task_kwargs) -> Any:
+                return (
+                    inner_task.func(*task_args, **task_kwargs)
+                    + inner_task.func(*task_args, **task_kwargs)
+                    + offset
+                )
+
+            return do_doubling
+
+        return _doubled_task
+
+    @doubled_task()
+    def plus_one(x: int):
+        return 1 + x
+
+    assert scheduler.run(plus_one(2)) == 6
+    assert plus_one.hash == "eb330208b0723c0c478899869da03eb18d20ef33"
+
+    assert plus_one.inner_task.hash == "16ebc98096f780d2fe24d0d6f56dfce29008f5a3"
+    wrapped_task = plus_one.wrapped_task
+    assert wrapped_task and wrapped_task.hash == "16ebc98096f780d2fe24d0d6f56dfce29008f5a3"
+    assert plus_one.signature.parameters["x"].annotation == int
+
+    @doubled_task()
+    @doubled_task()
+    def plus_two(x: int):
+        return 2 + x
+
+    assert scheduler.run(plus_two(2)) == 16
+
+    assert plus_two.inner_task.hash == "5636d44845593eb5845f2ced8d204f1295d26653"
+    assert plus_one.signature.parameters["x"].annotation == int
+
+
+def test_use_wrapper_signature(scheduler: Scheduler) -> None:
+    """
+    Users can change the signature of the wrapped task.
+    """
+
+    @wraps_task(use_wrapper_signature=True)
+    def doubled_task(inner_task: Task) -> Callable[[int, int], int]:
+        def do_doubling(x: int, offset: int) -> Any:
+            return inner_task.func(x) + inner_task.func(x) + offset
+
+        return do_doubling
+
+    @doubled_task
+    def plus_one(x: int):
+        return x + 1
+
+    assert scheduler.run(plus_one(3, 1)) == 9
+    assert plus_one.signature.parameters["x"].annotation == int
+    assert plus_one.signature.parameters["offset"].annotation == int
+
+
+def test_cache_options() -> None:
+    """All the methods for setting cache options get sanitized into enums."""
+
+    # Start with setting them on the method
+    @task(cache=False)
+    def task1():
+        return None
+
+    assert task1.get_task_option("cache_scope") == CacheScope.CSE
+
+    @task(cache=True)
+    def task2():
+        return None
+
+    assert task2.get_task_option("cache_scope") == CacheScope.BACKEND
+
+    @task(cache_scope="CSE")
+    def task3():
+        return None
+
+    assert task3.get_task_option("cache_scope") == CacheScope.CSE
+
+    @task(cache_scope=CacheScope.CSE)
+    def task4():
+        return None
+
+    assert task4.get_task_option("cache_scope") == CacheScope.CSE
+
+    @task()
+    def task5():
+        return None
+
+    # And retest with option overrides.
+    assert task5.options(cache=False).get_task_option("cache_scope") == CacheScope.CSE
+    assert task5.options(cache=True).get_task_option("cache_scope") == CacheScope.BACKEND
+    assert task5.options(cache_scope="CSE").get_task_option("cache_scope") == CacheScope.CSE
+    assert (
+        task5.options(cache_scope=CacheScope.CSE).get_task_option("cache_scope") == CacheScope.CSE
+    )
+    with pytest.raises(ValueError, match="'typo' is not a valid CacheScope"):
+        task5.options(cache_scope="typo")
+
+    assert task5.options(check_valid="full").get_task_option("check_valid") == CacheCheckValid.FULL
+    assert (
+        task5.options(check_valid=CacheCheckValid.FULL).get_task_option("check_valid")
+        == CacheCheckValid.FULL
+    )
+    assert (
+        task5.options(check_valid="shallow").get_task_option("check_valid")
+        == CacheCheckValid.SHALLOW
+    )
+    assert (
+        task5.options(check_valid=CacheCheckValid.SHALLOW).get_task_option("check_valid")
+        == CacheCheckValid.SHALLOW
+    )
+    with pytest.raises(ValueError, match="'typo' is not a valid CacheCheckValid"):
+        task5.options(check_valid="typo")
+
+    # Check the enum will traverse serialization.
+    type_registry = get_type_registry()
+    deserialized = type_registry.deserialize(
+        "redun.Task",
+        type_registry.serialize(task4.options(cache_scope=CacheScope.NONE)),
+    )
+    assert deserialized.get_task_option("cache_scope") == CacheScope.NONE
